import pandas as pd
import math

df = pd.read_excel("和訳評価_TED_42604_eval.xlsx")

system_outputs = []
human_scores = []

# for system 1
for i in range(1, 5):
    for id in set(df['原文\n番号']):
        cur_id_data = df.loc[df['原文\n番号'] == id]
        system_outputs.append(list(cur_id_data[f'System{i}'])[0]+'\n')
        if i == 1:
            score = -sum(list(cur_id_data['エラー点数'].dropna()))
        else:
            score = -sum(list(cur_id_data[f'エラー点数{i}'].dropna()))

        human_scores.append(str(score)+'\n')

src_ref_lines = open('ted_42604.en-ja-fixed.txt', 'r').readlines()
src_lines = [line.split('\t')[0]+'\n' for line in src_ref_lines]*4
ref_lines = [line.split('\t')[1] for line in src_ref_lines]*4

en_ja_src_file = open('mqm_en_ja_src.txt', 'w')
en_ja_ref_file = open('mqm_en_ja_ref.txt', 'w')
en_ja_sys_file = open('mqm_en_ja_sys.txt', 'w')
en_ja_scores_file = open('mqm_en_ja_scores.txt', 'w')

en_ja_src_file.writelines(src_lines)
en_ja_ref_file.writelines(ref_lines)
en_ja_sys_file.writelines(system_outputs)
en_ja_scores_file.writelines(human_scores)

print("files are saved!")



import glob
import click
import json

"""This file is used to convert all the post processed swappy files into txt and it produces Zh-En: 236,229,947 pair, En-De: 145,126,334 pairs"""
"""code to run this file: python3 txt_gen.py -dir_addr 6434175 -src_lang zh -tar_lang zh"""

@click.command()
@click.option('-dir_addr')
@click.option('-src_lang')
@click.option('-tar_lang')
@click.option('-count')
def main(dir_addr, src_lang, tar_lang, count):
    if count == '2M':
        slice_end = 2000000
    elif count == '4M':
        slice_end = 4000000
    else:
        print("Current count is not supported!")

    raw_file_ls = sorted(glob.glob(f'{dir_addr}/*'), reverse=True)
    global_count = 0
    saveSrcFile = open(f"{src_lang}_{count}.txt", 'w')
    saveRefFile = open(f"{tar_lang}_{count}.txt", 'w')

    for f_name in raw_file_ls:
        with open(f_name, 'r') as json_file:
            print(f_name)
            json_list = list(json_file)
            if global_count >= slice_end:
                break
            else:
                for json_str in json_list:
                    if json_str.strip():
                        result = json.loads(str(json_str))
                        # sanity check
                        if result['src_lang'] == src_lang and result['trg_lang'] == tar_lang:
                            saveSrcFile.write(result['src_text']+'\n')
                            saveRefFile.write(result['trg_text']+'\n')
                            global_count+=1

    print(f"All data is saved at {src_lang}_{count}.txt and {tar_lang}_{count}.txt!")

if __name__ == "__main__":
    main()



import click
import random
from syn_data.biimt import construct_uniform_mask_data

"""python3 syn_data/gen_eval_data.py -src_lang zh -tar_lang en -file_type dev -quantity 2M -mask_ratio 0.7 -aug_factor 2"""

@click.command()
@click.option('-src_lang')
@click.option('-tar_lang')
@click.option('-file_type', help="dev or train")
@click.option('-quantity', help='2M or 4M')
@click.option('-mask_ratio', type=float)
@click.option('-aug_factor', type=int)
@click.option('-prefix', type=str, default=None)
def main(src_lang, tar_lang, file_type, quantity, mask_ratio, aug_factor, prefix):
    if not prefix:
        src_dev_data = open(f'mt_infilling_data/{src_lang}-{tar_lang}/{quantity}/{src_lang}_{file_type}_{quantity}.txt', 'r').readlines()
        ref_dev_data = open(f'mt_infilling_data/{src_lang}-{tar_lang}/{quantity}/{tar_lang}_{file_type}_{quantity}.txt', 'r').readlines()
    else:
        if prefix == 'news':
            src_dev_data = open(f'news_data/{src_lang}-{tar_lang}/news_comp_{src_lang}_{tar_lang}.{src_lang}_dev.txt')
            ref_dev_data = open(f'news_data/{src_lang}-{tar_lang}/news_comp_{src_lang}_{tar_lang}.{tar_lang}_dev.txt')
        else:
            print("We currently only supports data in news domain")
            exit(1)

    mask_inp_dev, mask_ref_dev = construct_uniform_mask_data(src_dev_data, ref_dev_data, '<mask>', '<sep>', mask_ratio, aug_factor)
    mask_inp_dev, mask_ref_dev = [sen+'\n' for sen in mask_inp_dev], [sen+'\n' for sen in mask_ref_dev]
    mask_src_file = open(f'{prefix}_{src_lang}_{tar_lang}_mask_src_{mask_ratio}_{aug_factor}.txt', 'w')
    mask_ref_file = open(f'{prefix}_{src_lang}_{tar_lang}_mask_tar_{mask_ratio}_{aug_factor}.txt', 'w')
    mask_src_file.writelines(mask_inp_dev)
    mask_ref_file.writelines(mask_ref_dev)

    print("Save masked dev data!")

if __name__ == "__main__":
    random.seed(10)
    main()



import torch
import click
import nltk
from syn_data.biimt import exp_config, preprocess_data
from transformers import (
    AutoTokenizer
)
import random
from train.cl_train import exp_config
import math

"""In the synthetic data generation, we want MT infilling model to create five
levels of outputss. Sampling by five non-overlap masks (Each span is 15% of sentences) codes:
CUDA_VISIBLE_DEVICES=0 python3 syn_data/syn_data_gen.py -model_checkpoint mt_infilling_weights_zh-en_2M_1660497817/epoch4.ckpt \
-src_file news_prepare_mask_data_syn_gen/cycle1_zh_en_news_train_src_mask.txt -ref_file news_prepare_mask_data_syn_gen/cycle1_zh_en_news_train_ref_mask.txt"""

"""Yield batch sized list of sentences."""
def batchify(lst, batch_size):
    for i in range(0, len(lst), batch_size):
        yield lst[i:min(i+batch_size, len(lst))]

@click.command()
@click.option('-gpu_index', type=int)
@click.option('-prefix', type=str)
@click.option('-batch_size', type=int, default=64)
@click.option('-model_checkpoint', type=str)
@click.option('-tok_checkpoint', type=str, default="mbart-large-50-many-to-one-mmt-tok")
@click.option('-src_file', type=str, help="addr to the src file", default='news_comp_zh_en.zh_train.txt')
@click.option('-ref_file', type=str, help="addr to the ref file", default='news_comp_zh_en.en_train.txt')
@click.option('-save_dir', type=str)
def main(gpu_index, prefix, batch_size, model_checkpoint, tok_checkpoint, src_file, ref_file, save_dir):
    torch.cuda.empty_cache()
    device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
    tokenizer = AutoTokenizer.from_pretrained(tok_checkpoint)
    num_added_toks = tokenizer.add_special_tokens({'additional_special_tokens': ['<sep>']})
    model = torch.load(model_checkpoint).to(device)
    model.eval()
    # load in dev data
    mask_inp, mask_ref = open(src_file, 'r').readlines(), open(ref_file, 'r').readlines()
    mask_inp, mask_ref = [sen[:-1] for sen in mask_inp], [sen[:-1] for sen in mask_ref]
    # determine the start line and end line index for current gpu
    total_num_gpus = 8
    cur_gpu_size = math.ceil(len(mask_inp)/total_num_gpus)
    line_index_ls = list(range(0, len(mask_inp), cur_gpu_size))
    start_index, end_index = line_index_ls[gpu_index], min(line_index_ls[gpu_index]+cur_gpu_size, len(mask_inp))
    # rewrite the mask_inp into current gpu scope
    mask_inp, mask_ref = mask_inp[start_index:end_index], mask_ref[start_index:end_index]

    dataloader = preprocess_data(mask_inp, mask_ref, tokenizer, exp_config.max_length, batch_size, shuffle=False, sampler=False)

    gen_file = open(f'{save_dir}/{prefix}_{start_index}_{end_index}.outputs', 'w')
    with torch.no_grad():
        for data in dataloader:
            translated_tokens = model.generate(data['input_ids'].to(device))
            batch_texts = tokenizer.batch_decode(translated_tokens, skip_special_tokens=True)
            for text in batch_texts:
                gen_file.write(text+'\n')
    print(f"All outputs are saved at {prefix}.outputs")

if __name__ == "__main__":
    main()



import click

@click.command()
@click.option('-file_name')
@click.option('-count')
@click.option('-type')
def main(file_name, count, type):
    lines_ls = open(file_name).readlines()
    lines_ls = [line[:-1] for line in lines_ls]
    en_file = open(f"en_{type}_{count}.txt", 'w')
    de_file = open(f"de_{type}_{count}.txt", 'w')
    for line in lines_ls:
        line_ele_ls = line.split('\t<b>\t</b>\t')
        de_line, en_line = line_ele_ls[0], line_ele_ls[1]
        en_file.write(en_line+'\n')
        de_file.write(de_line+'\n')
    print("All lines are processed!")

if __name__ == "__main__":
    main()



from transformers import (
    XLMRobertaTokenizer,
    AutoModelForMaskedLM
)
import torch
import click
import os
import numpy as np
import time
import glob

"""code to run: python3 syn_data/phrase_mining.py -file_addr wmt_news/wmt_news_zh_en.txt -batch_size 256 -start_index 0 -end_index 10000 -lang_dir zh-en"""

"""Yield batch sized list of sentences."""
def batchify(lst, batch_size):
    for i in range(0, len(lst), batch_size):
        yield lst[i:min(i+batch_size, len(lst))]

# 1) Use xlm to derive the sentence embeddings and compute L2 on pairs of sentences
# 2) Select top 1000 sentences for each instance
# 3) For each instance, construct phrase-level emebddings in 1-6 window sizes and stride size 1

def sent_emb(hidden_states, emb_type, attention_mask):
    if emb_type == 'last_layer':
        sen_embed = (hidden_states[-1]*attention_mask.unsqueeze(-1)).sum(1)/attention_mask.sum(-1).unsqueeze(-1)
    elif emb_type == 'avg_first_last':
        sen_embed = ((hidden_states[-1]+hidden_states[0])/2.0*attention_mask.unsqueeze(-1)).sum(1)/attention_mask.sum(-1).unsqueeze(-1)
    else:
        print(f"{emb_type} sentence emb type is not supported!")
        exit(1)
    return sen_embed

def pool(model, encoded_input, attention_mask, emb_type, device):
    encoded_input, attention_mask = encoded_input.to(device), attention_mask.to(device)
    outputs = model(input_ids=encoded_input, attention_mask=attention_mask, output_hidden_states=True)
    pool_embed = sent_emb(outputs.hidden_states, emb_type, attention_mask)
    return pool_embed

@click.command()
@click.option('-file_addr', type=str)
@click.option('-batch_size', type=int)
@click.option('-lang_dir', type=str)
@click.option('-start_index', type=int)
@click.option('-end_index', type=int)
@click.option('-folder_name', type=str)
def main(file_addr, batch_size, lang_dir, start_index, end_index, folder_name):
    emb_type, max_length = 'last_layer', 256
    lines = open(file_addr, 'r').readlines()[start_index:end_index]
    lines = [line[:-1] for line in lines]
    # load in xlm models and tokenizers
    device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
    model = AutoModelForMaskedLM.from_pretrained("xlm-roberta-large-model").to(device)
    model.eval()
    tokenizer = XLMRobertaTokenizer.from_pretrained("xlm-roberta-large-tok")

    save_pair_indices = [[i, min(i+batch_size, len(lines))] for i in range(0, len(lines), batch_size)]

    if not os.path.isdir(folder_name):
        os.makedirs(folder_name)

    start_time = time.time()
    with torch.no_grad():
        for batch_text, save_pair_indice in zip(batchify(lines, batch_size), save_pair_indices):
            batch_dict = tokenizer(batch_text, return_tensors="pt", max_length=max_length, truncation=True, padding='max_length')
            batch_embed = pool(model, batch_dict['input_ids'], batch_dict['attention_mask'], emb_type, device)
            # save the current batch embeddings
            batch_embed = batch_embed.cpu().detach().numpy()
            save_start_index, save_end_index = save_pair_indice[0]+start_index, save_pair_indice[1]+start_index
            with open(f'{folder_name}/{save_start_index}_{save_end_index}.npy', 'wb') as f:
                np.save(f, batch_embed)
    end_time = time.time()
    print(f"Time duration: {end_time-start_time}s")

    # finish generating all sentence embeddings for the sentence
    files_ls = glob.glob(f'{folder_name}/*')
    sorted_file_ls = sorted(files_ls, key=lambda x:int(x.split('/')[1].split('_')[0]))
    total_ls =[]

    for file_name in sorted_file_ls:
        cur_ls = np.load(file_name).tolist()
        total_ls.extend(cur_ls)

    save_np_name=f"{folder_name}/{start_index}_total.npy"
    with open(save_np_name, 'wb') as f:
        np.save(f, np.array(total_ls))
    print("merge all the files!")

if __name__ == "__main__":
    main()



import random
import click

@click.command()
@click.option('-inp_file')
def main(inp_file):
    inp_lines = open(inp_file, 'r').readlines()
    score_file = open(f'pos_neg_{inp_file}', 'w')

    pos_count, neg_count = 0, 0
    for line_index, line in enumerate(inp_lines):
        line_ls = line[:-1].split('\t')
        ref_line = line_ls[0]
        # 0: pos, original sentence; 1: easy neg: random sentence; 2: hard neg
        random_select = random.choices([0,1,2], weights=[1/16, 1/16, 14/16], k=1)[0]

        if random_select == 0:
            score_file.write(ref_line+'\t'+ref_line+'\t'+str(0)+'\n')
            pos_count+=1
        elif random_select == 1:
            random_neg_index = random.choices(range(len(inp_lines)), k=1)[0]
            selected_sen = inp_lines[random_neg_index].split('\t')[0]
            if random_neg_index!=line_index:
                score_file.write(ref_line+'\t'+selected_sen+'\t'+str(-50)+'\n')
                neg_count+=1
        score_file.write(line)

    print(pos_count)
    print(neg_count)

if __name__ == "__main__":
    main()



import argparse
import os
import time
import numpy as np
from utils import *
from regression import *
import evaluate

SRC_HYPO = read_file_to_list('files/src_hypo_prompt.txt')
REF_HYPO = read_file_to_list('files/ref_hypo_prompt.txt')


class Scorer:
    """ Support ROUGE-1,2,L, BERTScore, MoverScore, PRISM, BARTScore """

    def __init__(self, file_path, device='cuda:0'):
        """ file_path: path to the pickle file
            All the data are normal capitalized, and tokenized, including ref_summs, and sys_summ.
        """
        self.device = device
        self.data = read_pickle(file_path)
        print(f'Data loaded from {file_path}.')

    def save_data(self, path):
        save_pickle(self.data, path)

    def score(self, metrics):
        """ metrics: list of metrics """
        for metric_name in metrics:
            if metric_name == 'bert_score':
                from bert_score import BERTScorer

                # Set up BERTScore
                bert_scorer = BERTScorer(
                    lang='en',
                    idf=False,
                    rescale_with_baseline=True,
                    device=self.device
                )
                print(f'BERTScore setup finished. Begin calculating BERTScore.')

                start = time.time()
                for doc_id in self.data:
                    ref_summs = self.data[doc_id]['ref_summs']
                    sys_summ = self.data[doc_id]['sys_summ']
                    P, R, F = bert_scorer.score([sys_summ] * len(ref_summs), ref_summs)
                    P = P.max().item()
                    R = R.max().item()
                    F = F.max().item()
                    self.data[doc_id]['scores']['bert_score_p'] = P
                    self.data[doc_id]['scores']['bert_score_r'] = R
                    self.data[doc_id]['scores']['bert_score_f'] = F
                print(f'Finished calculating BERTScore, time passed {time.time() - start}s.')

            elif metric_name == 'bleu':
                sacrebleu = evaluate.load("sacrebleu")
                start = time.time()
                for doc_id in self.data:
                    ref_summs = self.data[doc_id]['ref_summs']
                    sys_summ = self.data[doc_id]['sys_summ']
                    seg_scores = []
                    for out_line, ref_line in zip([sys_summ] * len(ref_summs), ref_summs):
                        results = sacrebleu.compute(predictions=[out_line], references=[[ref_line]])
                        seg_scores+= [results["score"]]
                    self.data[doc_id]['scores']['bleu'] = max(seg_scores)
                print(f'Finished calculating BLEU, time passed {time.time() - start}s.')

            elif metric_name == 'chrf':
                chrf = evaluate.load("chrf")
                start = time.time()
                for doc_id in self.data:
                    ref_summs = self.data[doc_id]['ref_summs']
                    sys_summ = self.data[doc_id]['sys_summ']
                    seg_scores = []
                    for out_line, ref_line in zip([sys_summ] * len(ref_summs), ref_summs):
                        results = chrf.compute(predictions=[out_line], references=[[ref_line]])
                        seg_scores+= [results["score"]]
                    self.data[doc_id]['scores']['chrf'] = max(seg_scores)
                print(f'Finished calculating chrF, time passed {time.time() - start}s.')

            elif metric_name == 'ter':
                seg_scores=[]
                ter = evaluate.load("ter")
                start = time.time()
                for doc_id in self.data:
                    ref_summs = self.data[doc_id]['ref_summs']
                    sys_summ = self.data[doc_id]['sys_summ']
                    seg_scores = []
                    for out_line, ref_line in zip([sys_summ] * len(ref_summs), ref_summs):
                        results = ter.compute(predictions=[out_line], references=[[ref_line]], case_sensitive=True)
                        seg_scores+= [results["score"]]
                    self.data[doc_id]['scores']['ter'] = max(seg_scores)
                print(f'Finished calculating TER, time passed {time.time() - start}s.')

            elif metric_name == 'bleurt':
                from bleurt import score
                checkpoint = "bleurt/bleurt-large-128"
                scorer = score.BleurtScorer(checkpoint)
                start = time.time()
                for doc_id in self.data:
                    ref_summs = self.data[doc_id]['ref_summs']
                    sys_summ = self.data[doc_id]['sys_summ']
                    scores_ls = scorer.score(references=ref_summs, candidates=[sys_summ] * len(ref_summs))
                    self.data[doc_id]['scores']['bleurt'] = max(scores_ls)
                print(f'Finished calculating bleurt, time passed {time.time() - start}s.')

            elif metric_name == 'sescore':
                from comet import load_from_checkpoint
                model = load_from_checkpoint('sescore_english.ckpt')
                for doc_id in self.data:
                    ref_summs = self.data[doc_id]['ref_summs']
                    sys_summ = self.data[doc_id]['sys_summ']
                    data = [{"src": ref_summs_ele, "mt": sys_summ_ele} for ref_summs_ele, sys_summ_ele in \
                    zip(ref_summs, [sys_summ] * len(ref_summs))]
                    scores_ls, _ = model.predict(data, batch_size=16, gpus=1)
                    self.data[doc_id]['scores']['sescore'] = max(scores_ls)
                print(f'Finished calculating SEScore, time passed {time.time() - start}s.')

            elif metric_name == 'contra_score':
                import torch
                exp_config.device_id = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
                model = torch.load('epoch0_best_7072_zh_en.ckpt').to(exp_config.device_id)
                model.eval()
                tokenizer = AutoTokenizer.from_pretrained("xlm-roberta-large-tok")

                start = time.time()
                for doc_id in self.data:
                    ref_summs = self.data[doc_id]['ref_summs']
                    sys_summ = self.data[doc_id]['sys_summ']
                    cur_data_dict = {'pivot': ref_summs, 'mt': [sys_summ] * len(ref_summs)}
                    cur_data_loader = preprocess_data(cur_data_dict, tokenizer, exp_config.max_length, len(ref_summs), shuffle=False, sampler=False, mode='test')
                    scores_ls = []
                    for batch in cur_data_loader:
                        score = model(batch, 'last_layer').squeeze(1).tolist()
                        scores_ls.extend(score)
                    self.data[doc_id]['scores']['contra_score'] = max(scores_ls)
                print(f'Finished calculating ContraScore, time passed {time.time() - start}s.')

            elif metric_name == 'mover_score':
                from moverscore import word_mover_score, get_idf_dict

                # Set up MoverScore
                self.stop_words = []

                ref_lines = []
                for doc_id in self.data:
                    ref_lines.extend(self.data[doc_id]['ref_summs'])
                ref_lines = list(set(ref_lines))
                self.idf_refs = get_idf_dict(ref_lines)

                # IDF for all system hypos, used for MoverScore
                sys_lines = []
                for doc_id in self.data:
                    sys_summ = self.data[doc_id]['sys_summ']
                    sys_lines.append(sys_summ)
                self.idf_hyps = get_idf_dict(sys_lines)
                print(f'MoverScore setup finished. Begin calculating MoverScore.')

                start = time.time()
                for doc_id in self.data:
                    ref_summs = self.data[doc_id]['ref_summs']
                    sys_summ = self.data[doc_id]['sys_summ']
                    scores = word_mover_score(ref_summs, [sys_summ] * len(ref_summs), self.idf_refs, self.idf_hyps,
                                              self.stop_words,
                                              n_gram=1, remove_subwords=True, batch_size=48, device=self.device)
                    score = max(scores)
                    self.data[doc_id]['scores']['mover_score'] = score
                print(f'Finished calculating MoverScore, time passed {time.time() - start}s.')

            elif metric_name == 'rouge':
                from gehrmann_rouge_opennmt.rouge_baselines.baseline import baseline_main

                def rouge(dic):
                    """ Get r, p, f scores """
                    r1_, r2_, rl_ = [], [], []
                    for k in dic:
                        r1_.append([dic[k]['rouge_1_recall'], dic[k]['rouge_1_precision'], dic[k]['rouge_1_f_score']])
                        r2_.append([dic[k]['rouge_2_recall'], dic[k]['rouge_2_precision'], dic[k]['rouge_2_f_score']])
                        rl_.append([dic[k]['rouge_l_recall'], dic[k]['rouge_l_precision'], dic[k]['rouge_l_f_score']])
                    return r1_, r2_, rl_

                print(f'Begin calculating ROUGE.')
                start = time.time()
                blockPrint()
                for doc_id in self.data:
                    ref_summs = self.data[doc_id]['ref_summs']
                    sys_summ = self.data[doc_id]['sys_summ']
                    sys_summ = sys_summ.lower()
                    write_list_to_file([sys_summ] * len(ref_summs), 'hypo.txt')
                    write_list_to_file(ref_summs, 'ref.txt')
                    args = argparse.Namespace(check_repeats=True, delete=True, get_each_score=True, stemming=True,
                                              method='sent_no_tag', n_bootstrap=1000, run_google_rouge=False,
                                              run_rouge=True, source='./hypo.txt', target='./ref.txt',
                                              ref_sep='||NEVER||', num_ref=1, temp_dir='./temp/')

                    scores = baseline_main(args, return_pyrouge_scores=True)['individual_score_results']
                    r1, r2, rl = rouge(scores)
                    r1 = np.max(r1, axis=0)
                    r2 = np.max(r2, axis=0)
                    rl = np.max(rl, axis=0)

                    self.data[doc_id]['scores']['rouge1_r'] = r1[0]
                    self.data[doc_id]['scores']['rouge1_p'] = r1[1]
                    self.data[doc_id]['scores']['rouge1_f'] = r1[2]
                    self.data[doc_id]['scores']['rouge2_r'] = r2[0]
                    self.data[doc_id]['scores']['rouge2_p'] = r2[1]
                    self.data[doc_id]['scores']['rouge2_f'] = r2[2]
                    self.data[doc_id]['scores']['rougel_r'] = rl[0]
                    self.data[doc_id]['scores']['rougel_p'] = rl[1]
                    self.data[doc_id]['scores']['rougel_f'] = rl[2]
                enablePrint()
                os.system('rm -rf hypo.txt ref.txt saved_out.txt')
                print(f'Finished calculating ROUGE, time passed {time.time() - start}s.')

            elif metric_name == 'prism':
                from prism import Prism
                # Set up Prism
                self.prism = Prism(model_dir='./models/m39v1/', lang='en')
                print(f'PRISM setup finished. Begin calculating PRISM.')

                start = time.time()
                for doc_id in self.data:
                    ref_summs = self.data[doc_id]['ref_summs']
                    ref_summs = [detokenize(line) for line in ref_summs]
                    sys_summ = detokenize(self.data[doc_id]['sys_summ'])
                    ref_hypo_scores, hypo_ref_scores, scores = self.prism.score(cand=[sys_summ] * len(ref_summs),
                                                                                ref=ref_summs,
                                                                                segment_scores=True)
                    ref_hypo, hypo_ref, score = max(ref_hypo_scores), max(hypo_ref_scores), max(scores)
                    self.data[doc_id]['scores']['prism_ref_hypo'] = ref_hypo
                    self.data[doc_id]['scores']['prism_hypo_ref'] = hypo_ref
                    self.data[doc_id]['scores']['prism_avg'] = score
                print(f'Finished calculating PRISM, time passed {time.time() - start}s.')

            elif metric_name == 'bart_score' or metric_name == 'bart_score_cnn' or metric_name == 'bart_score_para':
                """ Vanilla BARTScore, BARTScore-CNN, BARTScore-CNN-Para """
                from bart_score import BARTScorer

                # Set up BARTScore
                if 'cnn' in metric_name:
                    bart_scorer = BARTScorer(device=self.device, checkpoint='facebook/bart-large-cnn')
                elif 'para' in metric_name:
                    bart_scorer = BARTScorer(device=self.device, checkpoint='facebook/bart-large-cnn')
                    bart_scorer.load()
                else:
                    bart_scorer = BARTScorer(device=self.device, checkpoint='facebook/bart-large')
                print(f'BARTScore setup finished. Begin calculating BARTScore.')

                start = time.time()
                for doc_id in self.data:
                    ref_summs = self.data[doc_id]['ref_summs']
                    ref_summs = [detokenize(line) for line in ref_summs]
                    sys_summ = detokenize(self.data[doc_id]['sys_summ'])

                    ref_hypo_scores = np.array(bart_scorer.score(ref_summs, [sys_summ] * len(ref_summs), batch_size=4))
                    hypo_ref_scores = np.array(bart_scorer.score([sys_summ] * len(ref_summs), ref_summs, batch_size=4))
                    ref_hypo = ref_hypo_scores.max()
                    hypo_ref = hypo_ref_scores.max()
                    avg_f = (0.5 * (ref_hypo_scores + hypo_ref_scores)).max()
                    harm_f = (ref_hypo_scores * hypo_ref_scores / (ref_hypo_scores + hypo_ref_scores)).max()

                    self.data[doc_id]['scores'][f'{metric_name}_ref_hypo'] = ref_hypo
                    self.data[doc_id]['scores'][f'{metric_name}_hypo_ref'] = hypo_ref
                    self.data[doc_id]['scores'][f'{metric_name}_avg_f'] = avg_f
                    self.data[doc_id]['scores'][f'{metric_name}_harm_f'] = harm_f
                print(f'Finished calculating BARTScore, time passed {time.time() - start}s.')

            elif metric_name.startswith('prompt'):
                """ BARTScore adding prompts """
                from bart_score import BARTScorer

                def prefix_prompt(l, p):
                    new_l = []
                    for x in l:
                        new_l.append(p + ', ' + x)
                    return new_l

                def suffix_prompt(l, p):
                    new_l = []
                    for x in l:
                        new_l.append(x + ' ' + p + ',')
                    return new_l

                if 'cnn' in metric_name:
                    name = 'bart_score_cnn'
                    bart_scorer = BARTScorer(device=self.device, checkpoint='facebook/bart-large-cnn')
                elif 'para' in metric_name:
                    name = 'bart_score_para'
                    bart_scorer = BARTScorer(device=self.device, checkpoint='facebook/bart-large-cnn')
                    bart_scorer.load()
                else:
                    name = 'bart_score'
                    bart_scorer = BARTScorer(device=self.device, checkpoint='facebook/bart-large')

                print(f'BARTScore-P setup finished. Begin calculating BARTScore-P.')
                start = time.time()
                for doc_id in self.data:
                    ref_summs = self.data[doc_id]['ref_summs']
                    ref_summs = [detokenize(line) for line in ref_summs]
                    sys_summ = detokenize(self.data[doc_id]['sys_summ'])
                    sys_summs = [sys_summ] * len(ref_summs)
                    for prompt in REF_HYPO:
                        ref_hypo_scores_en = np.array(
                            bart_scorer.score(suffix_prompt(ref_summs, prompt), sys_summs, batch_size=4))
                        hypo_ref_scores_en = np.array(
                            bart_scorer.score(suffix_prompt(sys_summs, prompt), ref_summs, batch_size=4))
                        ref_hypo_scores_de = np.array(
                            bart_scorer.score(ref_summs, prefix_prompt(sys_summs, prompt), batch_size=4))
                        hypo_ref_scores_de = np.array(
                            bart_scorer.score(sys_summs, prefix_prompt(ref_summs, prompt), batch_size=4))
                        ref_hypo_en = ref_hypo_scores_en.max()
                        hypo_ref_en = hypo_ref_scores_en.max()
                        avg_f_en = (0.5 * (ref_hypo_scores_en + hypo_ref_scores_en)).max()
                        harm_f_en = (ref_hypo_scores_en * hypo_ref_scores_en / (
                                ref_hypo_scores_en + hypo_ref_scores_en)).max()
                        ref_hypo_de = ref_hypo_scores_de.max()
                        hypo_ref_de = hypo_ref_scores_de.max()
                        avg_f_de = (0.5 * (ref_hypo_scores_de + hypo_ref_scores_de)).max()
                        harm_f_de = (ref_hypo_scores_de * hypo_ref_scores_de / (
                                ref_hypo_scores_de + hypo_ref_scores_de)).max()

                        self.data[doc_id]['scores'][f'{name}_ref_hypo_en_{prompt}'] = ref_hypo_en
                        self.data[doc_id]['scores'][f'{name}_hypo_ref_en_{prompt}'] = hypo_ref_en
                        self.data[doc_id]['scores'][f'{name}_avg_f_en_{prompt}'] = avg_f_en
                        self.data[doc_id]['scores'][f'{name}_harm_f_en_{prompt}'] = harm_f_en
                        self.data[doc_id]['scores'][f'{name}_ref_hypo_de_{prompt}'] = ref_hypo_de
                        self.data[doc_id]['scores'][f'{name}_hypo_ref_de_{prompt}'] = hypo_ref_de
                        self.data[doc_id]['scores'][f'{name}_avg_f_de_{prompt}'] = avg_f_de
                        self.data[doc_id]['scores'][f'{name}_harm_f_de_{prompt}'] = harm_f_de
                print(f'Finished calculating BARTScore, time passed {time.time() - start}s.')

            else:
                raise NotImplementedError


def main():
    parser = argparse.ArgumentParser(description='Scorer parameters')
    parser.add_argument('--file', type=str, required=True,
                        help='The data to load from.')
    parser.add_argument('--device', type=str, default='cuda:0',
                        help='The device to run on.')
    parser.add_argument('--output', type=str, required=True,
                        help='The output path to save the calculated scores.')
    parser.add_argument('--bert_score', action='store_true', default=False,
                        help='Whether to calculate BERTScore')
    parser.add_argument('--mover_score', action='store_true', default=False,
                        help='Whether to calculate MoverScore')
    parser.add_argument('--rouge', action='store_true', default=False,
                        help='Whether to calculate ROUGE')
    parser.add_argument('--bart_score', action='store_true', default=False,
                        help='Whether to calculate BARTScore')
    parser.add_argument('--bart_score_cnn', action='store_true', default=False,
                        help='Whether to calculate BARTScore-CNN')
    parser.add_argument('--bart_score_para', action='store_true', default=False,
                        help='Whether to calculate BARTScore-Para')
    parser.add_argument('--prism', action='store_true', default=False,
                        help='Whether to calculate PRISM')
    parser.add_argument('--contra_score', action='store_true', default=False,
                        help='Whether to calculate contra_score')
    parser.add_argument('--bleurt', action='store_true', default=False,
                        help='Whether to calculate bleurt')
    parser.add_argument('--bleu', action='store_true', default=False,
                        help='Whether to calculate bleu')
    parser.add_argument('--chrf', action='store_true', default=False,
                        help='Whether to calculate chrf')
    parser.add_argument('--ter', action='store_true', default=False,
                        help='Whether to calculate ter')
    parser.add_argument('--prompt', type=str, default=None,
                        help='Whether to calculate BARTScore-P. Can be bart_ref, '
                             'bart_cnn_ref, bart_para_ref')
    args = parser.parse_args()

    scorer = Scorer(args.file, args.device)

    METRICS = []
    if args.bert_score:
        METRICS.append('bert_score')
    if args.mover_score:
        METRICS.append('mover_score')
    if args.rouge:
        METRICS.append('rouge')
    if args.bart_score:
        METRICS.append('bart_score')
    if args.bart_score_cnn:
        METRICS.append('bart_score_cnn')
    if args.bart_score_para:
        METRICS.append('bart_score_para')
    if args.prism:
        METRICS.append('prism')
    if args.contra_score:
        METRICS.append('contra_score')
    if args.bleurt:
        METRICS.append('bleurt')
    if args.bleu:
        METRICS.append('bleu')
    if args.chrf:
        METRICS.append('chrf')
    if args.ter:
        METRICS.append('ter')
    if args.prompt is not None:
        prompt = args.prompt
        assert prompt in ['bart_ref', 'bart_cnn_ref', 'bart_para_ref']
        METRICS.append(f'prompt_{prompt}')

    scorer.score(METRICS)
    scorer.save_data(args.output)


if __name__ == '__main__':
    main()



from nltk.tokenize import word_tokenize
from nltk.tokenize.treebank import TreebankWordDetokenizer
import click
import spacy

"""code to run:
python3 syn_data/final_data_construct.py -lang english -severe_enable True -folder_name severity_measure_sep17_zh_en_news_0_5027365 -ref_addr sep17_zh_en_ssl_data/train.en """

def file_to_dict(syn_data_dict, file_name):
    inp_file = open(file_name, 'r').readlines()
    for line in inp_file:
        line_ls = line[:-1].split('\t')
        if int(line_ls[0]) not in syn_data_dict:
            syn_data_dict[int(line_ls[0])] = {}
            syn_data_dict[int(line_ls[0])]['span'] = []
            syn_data_dict[int(line_ls[0])]['content'] = []
            syn_data_dict[int(line_ls[0])]['score'] = []
        if line_ls[1] == '1':
            syn_data_dict[int(line_ls[0])]['span'].append([int(line_ls[2]), int(line_ls[2])])
        else:
            syn_data_dict[int(line_ls[0])]['span'].append([int(line_ls[2].split('_')[0]), int(line_ls[2].split('_')[1])])
        syn_data_dict[int(line_ls[0])]['content'].append(line_ls[3])
        syn_data_dict[int(line_ls[0])]['score'].append(float(line_ls[4]))
    return syn_data_dict

def sen_construct(locs_records, replaced_contents, words_ls):
    new_words_ls = []
    sorted_locs_cont = sorted(list(zip(locs_records, replaced_contents)))
    if len(sorted_locs_cont) == 1:
        new_words_ls += words_ls[:locs_records[0][0]]+replaced_contents+words_ls[locs_records[0][1]:]
    else:
        for cur_index, locs_content in enumerate(sorted_locs_cont):
            locs_pair, content = locs_content[0], locs_content[1]
            if cur_index == 0:
                new_words_ls += words_ls[:locs_pair[0]]+[content]
            elif cur_index == len(sorted_locs_cont)-1:
                new_words_ls += words_ls[sorted_locs_cont[cur_index-1][0][1]:locs_pair[0]]+[content]+words_ls[locs_pair[1]:]
            else:
                new_words_ls += words_ls[sorted_locs_cont[cur_index-1][0][1]:locs_pair[0]]+[content]
    return new_words_ls

@click.command()
@click.option('-lang', help="english")
@click.option('-severe_enable', help="If enabled, severity measure will be added!", type=bool)
@click.option('-folder_name')
@click.option('-ref_addr')
@click.option('-del_enable', type=bool)
@click.option('-insert_replace_enable', type=bool)
def main(lang, severe_enable, folder_name, ref_addr, del_enable, insert_replace_enable):
    syn_data_dict = {}

    for i in range(1, 6):
        if del_enable:
            syn_data_dict = file_to_dict(syn_data_dict, f'{folder_name}/score_del_sen_op_loc_{i}.txt')

    print(f"len of del: {len(syn_data_dict)}")

    for i in range(1, 6):
        if insert_replace_enable:
            syn_data_dict = file_to_dict(syn_data_dict, f'{folder_name}/scores_{lang}_{i}_mined_cont_rand_None.txt')

    print(f"len of del+insert+rep: {len(syn_data_dict)}")

    if len(syn_data_dict)==0:
        print("Error in loading score files")
        exit(1)

    print(syn_data_dict[58992])

    ref_lines = open(ref_addr, 'r').readlines()
    ref_lines = [line[:-1] for line in ref_lines]
    saveFile = open(f'{lang}_{severe_enable}_severe_minor.txt', 'w')

    if lang == 'japanese':
        nlp = spacy.blank('ja')

    for sen_index in syn_data_dict:
        if lang == 'japanese':
            words_ls = [word.text for word in nlp(ref_lines[sen_index])]
            mt_out = sen_construct(syn_data_dict[sen_index]['span'], syn_data_dict[sen_index]['content'], words_ls)
        else:
            mt_out = sen_construct(syn_data_dict[sen_index]['span'], syn_data_dict[sen_index]['content'], word_tokenize(ref_lines[sen_index]))
        if lang == 'japanese':
            mt_out = ''.join(mt_out)
        else:
            mt_out = TreebankWordDetokenizer().detokenize(mt_out)
        if severe_enable:
            score = sum(syn_data_dict[sen_index]['score'])
        else:
            score = -len(syn_data_dict[sen_index]['score'])
        saveFile.write(ref_lines[sen_index]+'\t'+mt_out+'\t'+str(score)+'\n')

    print(f"Saved at {lang}_{severe_enable}_severe_minor.txt!")

if __name__ == "__main__":
    main()



import click
from mt_metrics_eval import data

human_mapping_dict = {
    "wmt21.news": {
        'en-de': ['refA', 'refD'],
        'en-ru': ['refB'],
        'zh-en': ['refA']
    },
    "wmt20": {
        'en-de': 'refb',
        'zh-en': 'refb'
    }
}

def laser_baseline():
    pass

@click.command()
@click.option('-baseline_type', help="laser1, laser2, sen_tr")
def main(baseline_type):
    evs = data.EvalSet(wmt, lang)
    srcs, mt_outs_dict, refs = evs.src, evs.sys_outputs, evs.std_ref
    if baseline_type == 'laser1':
        pass
    elif baseline_type == 'laser2':
        pass
    elif baseline_type == 'sen_tr':
        from sentence_transformers import SentenceTransformer, util
        model = SentenceTransformer('paraphrase-multilingual-MiniLM-L12-v2')
        model.encode([])
    else:
        print("Code is not currently supporting more baseline types")

if __name__ == "__main__":
    main()



import numpy as np
import faiss
import click
import time
from tqdm.auto import tqdm
import math

"""Yield batch sized list of sentences."""
def batchify(lst, batch_size):
    for i in range(0, len(lst), batch_size):
        yield lst[i:min(i+batch_size, len(lst))]

def file_to_numpy(value_file, dim):
    value = np.fromfile(value_file, dtype=np.float32, count=-1)
    value.resize(value.shape[0] // dim, dim)
    return value

"""python3 syn_data/knn_search.py"""

@click.command()
@click.option('-query_file')
@click.option('-value_file_1')
@click.option('-value_file_2')
@click.option('-value_file_3')
@click.option('-value_file_4')
@click.option('-suffix', default="part1")
def main(query_file, value_file_1, value_file_2, value_file_3, value_file_4, suffix):
    temp_ls = query_file.split('.')[0].split('_')
    query_start, query_end = temp_ls[-2], temp_ls[-1]
    ids_save_file = open(f"ids_query_{query_start}_{query_end}_value_{suffix}.txt", 'w')
    dists_save_file = open(f"dists_query_{query_start}_{query_end}_value_{suffix}.txt", 'w')
    sum_save_file = open(f"sum_query_{query_start}_{query_end}_value_{suffix}.txt", 'w')

    # map the cpu index to gpu (internal processing: only once-> cost around 9 mins)
    dim = 1024
    ngpus = faiss.get_num_gpus()
    cpu_index = faiss.IndexFlatIP(dim)
    gpu_index = faiss.index_cpu_to_all_gpus(cpu_index)

    # load in the query matrix
    query = file_to_numpy(query_file, dim)
    # load in the value matrix
    val_1 = file_to_numpy(value_file_1, dim)
    val_ls = [val_1]
    if value_file_2 != 'None':
        val_2 = file_to_numpy(value_file_2, dim)
        val_ls.append(val_2)
    if value_file_3 != 'None':
        val_3 = file_to_numpy(value_file_3, dim)
        val_ls.append(val_3)
    if value_file_4 != 'None':
        val_4 = file_to_numpy(value_file_4, dim)
        val_ls.append(val_4)
    value = np.concatenate(val_ls, axis=0)
    value = value/np.expand_dims(np.linalg.norm(value, axis=1), axis=1) # normalize the vectors
    print("load in value matrix")

    gpu_index.add(value)
    print("load in data into index table")

    # first find 2048 k-nearest neighbors
    topk, batch_size = 128, 128
    start = time.time()
    with tqdm(total=math.ceil(query.shape[0]/batch_size)) as pbar:
        for batch_query in batchify(query, batch_size):
            dists_ls, ids_ls = gpu_index.search(x=batch_query, k=topk)
            for dists, ids in zip(dists_ls, ids_ls):
                temp_ls_ids = [str(ele) for ele in ids]
                save_ids_line = '\t'.join(temp_ls_ids)+'\n'
                temp_ls_dists = [str(ele) for ele in dists]
                save_dists_line = '\t'.join(temp_ls_dists)+'\n'
                save_sum_dists_line = str(np.sum(dists))+'\n'
                ids_save_file.write(save_ids_line)
                dists_save_file.write(save_dists_line)
                sum_save_file.write(save_sum_dists_line)
            pbar.update(1)
    end = time.time()

    print(end-start)
    print("files are saved!")

if __name__ == "__main__":
    main()



import click
import torch
import torch.nn as nn
import wandb
from datasets import load_dataset
from torch.utils.data import DataLoader
from datasets import Dataset
from transformers import (
    AutoConfig,
    AutoTokenizer,
    AutoModelForMaskedLM,
    DataCollatorWithPadding,
    get_linear_schedule_with_warmup,
    AdamW
)
import math
from torch.nn.parallel import DistributedDataParallel as DDP
import torch.distributed as dist
import os

class exp_config():
    max_length = 256
    hidden_size = 1024
    temp = 0.05
    lr = 3e-05

class XLM_Roberta_MLP():
    """
    Load the XLM-Roberta large pretrained weights and at the same time perform concatenations
    We store additional projection matrix: projecting max_length * hidden state to hidden state, then pass to ReLU
    This model returns the outputs in (batch size * hidden states), no pooling is required!
    """
    def __init__(self, model_addr):
        super().__init__()
        self.xlm = AutoModelForMaskedLM.from_pretrained(model_addr)
        self.dense = nn.Linear(exp_config.max_length * exp_config.hidden_size, exp_config.hidden_size)
        self.activation = nn.ReLU() # Use RELU to extract useful features

    def forward(self, input_ids, attention_mask, **kwargs):
        outputs = self.xlm(input_ids=encoded_input, attention_mask=attention_mask, output_hidden_states=True)
        features = torch.flatten(outputs.hidden_states[-1], start_dim=1)
        x = self.dense(features)
        x = self.activation(x)

        return x

def sent_emb(hidden_states, emb_type, attention_mask):
    if emb_type == 'last_layer':
        sen_embed = (hidden_states[-1]*attention_mask.unsqueeze(-1)).sum(1)/attention_mask.sum(-1).unsqueeze(-1)
    elif emb_type == 'avg_first_last':
        sen_embed = ((hidden_states[-1]+hidden_states[0])/2.0*attention_mask.unsqueeze(-1)).sum(1)/attention_mask.sum(-1).unsqueeze(-1)
    elif emb_type == 'states_concat':
        sen_embed = outputs
    else:
        print(f"{emb_type} sentence emb type is not supported!")
    return sen_embed

def pool(model, encoded_input, attention_mask, emb_type):
    encoded_input, attention_mask = encoded_input.to(exp_config.device_id), attention_mask.to(exp_config.device_id)
    outputs = model(input_ids=encoded_input, attention_mask=attention_mask, output_hidden_states=True)
    pool_embed = sent_emb(outputs.hidden_states, emb_type, attention_mask)
    return pool_embed

class CL_XLM_Roberta(nn.Module):
    def __init__(self, model_addr):
        super().__init__()
        self.xlm = AutoModelForMaskedLM.from_pretrained(model_addr)

    def forward(self, batch, strategy, emb_type):
        src_pool_embed = pool(self.xlm, batch['input_ids'], batch['src_attn_masks'], emb_type)
        pos_pool_embed = pool(self.xlm, batch['tar'], batch['tar_attn_masks'], emb_type)

        if strategy == 'margin_src_ref_batch':
            neg_batch_embed = torch.cat((pos_pool_embed[1:, :], pos_pool_embed[:1, :]), dim=0)
            return src_pool_embed, pos_pool_embed, neg_batch_embed
        elif strategy == 'margin_src_ref_mt1':
            mt1_pool_embed = pool(self.xlm, batch['mt1'], batch['mt1_attn_masks'], emb_type)
            return src_pool_embed, pos_pool_embed, mt1_pool_embed
        elif strategy == 'margin_src_ref_mt1_batch':
            mt1_pool_embed = pool(self.xlm, batch['mt1'], batch['mt1_attn_masks'], emb_type)
            neg_batch_embed = torch.cat((pos_pool_embed[1:, :], pos_pool_embed[:1, :]), dim=0)
            return src_pool_embed, pos_pool_embed, mt1_pool_embed, neg_batch_embed
        else:
            print("Your training strategy is not supported!")
            exit(1)

class Similarity(nn.Module):
    """
    Dot product or cosine similarity
    """

    def __init__(self, temp):
        super().__init__()
        self.temp = temp
        self.cos = nn.CosineSimilarity(dim=-1)

    def forward(self, x, y):
        return -self.cos(x, y) / self.temp

def preprocess_data(src_data, ref_data, tokenizer, max_length, batch_size, mt1_data=None, shuffle=True, sampler=True):
    if mt1_data:
        ds = Dataset.from_dict({"src": src_data, 'tar': ref_data, 'mt1': mt1_data})
    else:
        ds = Dataset.from_dict({"src": src_data, 'tar': ref_data})

    def preprocess_function(examples):
        model_inputs = {}
        srcs = tokenizer(examples['src'], max_length=max_length, padding='max_length', truncation=True)
        model_inputs['input_ids'], model_inputs['src_attn_masks'] = srcs['input_ids'], srcs['attention_mask']
        targets = tokenizer(examples['tar'], max_length=max_length, padding='max_length', truncation=True)
        model_inputs['tar'], model_inputs['tar_attn_masks'] = targets["input_ids"], targets['attention_mask']
        if mt1_data:
            mt1_texts = tokenizer(examples['mt1'], max_length=max_length, padding='max_length', truncation=True)
            model_inputs['mt1'], model_inputs['mt1_attn_masks'] = mt1_texts["input_ids"], mt1_texts['attention_mask']
        return model_inputs

    processed_datasets = ds.map(
        preprocess_function,
        batched=True,
        num_proc=None,
        remove_columns=ds.column_names,
        desc="Running tokenizer on dataset",
    )

    data_collator = DataCollatorWithPadding(
        tokenizer=tokenizer,
        padding='max_length',
        max_length=max_length,
        pad_to_multiple_of=None,
        return_tensors = 'pt'
    )
    if sampler:
        data_sampler = torch.utils.data.distributed.DistributedSampler(processed_datasets, shuffle=shuffle)
        dataloader = DataLoader(processed_datasets, batch_size=batch_size, collate_fn=data_collator, sampler=data_sampler)
    else:
        dataloader = DataLoader(processed_datasets, batch_size=batch_size, collate_fn=data_collator, shuffle=shuffle)
    return dataloader

class loss_funct():
    def __init__(self, strategy, score_type):
        self.strategy = strategy
        if score_type == "cosine":
            self.score = Similarity(temp=exp_config.temp)
            margin = 0.01
        elif score_type == "L2":
            self.score = nn.PairwiseDistance(p=2)
            margin = 1
        else:
            print("We only support two distance metrics: cosine and L2")
            exit(1)

        # parameter order: anchor, positive, negative
        self.loss_fct_1 = nn.TripletMarginWithDistanceLoss(distance_function=self.score, margin=margin)
        if strategy == 'margin_src_ref_mt1_batch':
            self.loss_fct_2 = nn.TripletMarginWithDistanceLoss(distance_function=self.score, margin=margin*2)

    def loss_compute(self, train_batch, model, emb_type):
        if self.strategy == 'margin_src_ref_batch':
            src_pool_embed, pos_pool_embed, neg_batch_embed = model(train_batch, self.strategy, emb_type)
            return [self.loss_fct_1(src_pool_embed, pos_pool_embed, neg_batch_embed)]
        elif self.strategy == 'margin_src_ref_mt1':
            src_pool_embed, pos_pool_embed, mt1_pool_embed = model(train_batch, self.strategy, emb_type)
            return [self.loss_fct_1(src_pool_embed, pos_pool_embed, mt1_pool_embed)]
        elif self.strategy == 'margin_src_ref_mt1_batch':
            src_pool_embed, pos_pool_embed, mt1_pool_embed, neg_batch_embed = model(train_batch, self.strategy, emb_type)
            loss_1 = self.loss_fct_1(src_pool_embed, pos_pool_embed, mt1_pool_embed)
            loss_2 = self.loss_fct_2(src_pool_embed, pos_pool_embed, neg_batch_embed)
            loss_3 = self.loss_fct_1(pos_pool_embed, mt1_pool_embed, neg_batch_embed)
            return [loss_1, loss_2, loss_3]
        else:
            print("Your training strategy is not supported!")
            exit(1)

def store_cl_loss(model, dev_dataloader, loss_manager, train_batch, emb_type):
    model.eval()
    with torch.no_grad():
        train_loss_ls = loss_manager.loss_compute(train_batch, model, emb_type)
        train_loss = torch.stack(train_loss_ls, dim=0).sum(dim=0)
        loss_size = len(train_loss_ls)
        # store all training loss
        if loss_size == 3:
            dev_loss1, dev_loss2, dev_loss3 = 0, 0, 0
        dev_loss = 0
        # store all dev loss
        for dev_batch in dev_dataloader:
            dev_loss_ls = loss_manager.loss_compute(dev_batch, model, emb_type)
            if loss_size == 3:
                dev_loss1=dev_loss1+dev_loss_ls[0]
                dev_loss2=dev_loss2+dev_loss_ls[1]
                dev_loss3=dev_loss3+dev_loss_ls[2]
            dev_loss = dev_loss+torch.stack(dev_loss_ls, dim=0).sum(dim=0)

        dev_loss = dev_loss/len(dev_dataloader)

        if loss_size == 3:
            dev_loss1, dev_loss2, dev_loss3 = dev_loss1/len(dev_dataloader), dev_loss2/len(dev_dataloader), dev_loss3/len(dev_dataloader)
            wandb.log({"training loss (src, pos, mt1)": train_loss_ls[0].item(),
                       "training loss (src, pos, batchNeg)": train_loss_ls[1].item(),
                       "training loss (pos, mt, batchNeg)": train_loss_ls[2].item(),
                       "training loss": torch.stack(train_loss_ls, dim=0).sum(dim=0).item(),
                       "dev loss (src, pos, mt1)": dev_loss1.item(),
                       "dev loss (src, pos, batchNeg)": dev_loss2.item(),
                       "dev loss (pos, mt, batchNeg)": dev_loss3.item(),
                       "dev loss": dev_loss.item()
                      })
        else:
            wandb.log({
                       "training loss": torch.stack(train_loss_ls, dim=0).sum(dim=0).item(),
                       "dev loss": dev_loss.item()
                      })
    model.train()

@click.command()
@click.option('-gradient_accumulation_steps', default=1, type=int)
@click.option('-src_train_file', type=str, help="addr to the src train file")
@click.option('-ref_train_file', type=str, help="addr to the ref train file")
@click.option('-src_dev_file', type=str, help="addr to the src dev file")
@click.option('-ref_dev_file', type=str, help="addr to the ref dev file")
@click.option('-mt_file_train_1', type=str, help="addr to the first MT train output file", default=None)
@click.option('-mt_file_dev_1', type=str, help="addr to the first MT dev output file", default=None)
@click.option('-batch_size', type=int, help="batch size for contrastive learning")
@click.option('-emb_type', type=str, help="choose from last_layer, avg_first_last and states_concat")
@click.option('-strategy', type=str, help="choose from margin_src_ref_batch, margin_src_ref_mt1 and margin_src_ref_mt1_batch")
@click.option('-num_epoch', type=int, help="Number of epoches to train", default=5)
@click.option('-eval_step', type=int, help="Number of steps to evaluate", default=4)
@click.option('-num_warmup_steps', type=int, help="Number of steps to warm up", default=0)
@click.option('-save_dir_name', type=str, help="the dir name of weights being saved", default=None)
@click.option('-score_type', type=str, help="choose between L2 and cosine")
def main(src_train_file, ref_train_file, src_dev_file, ref_dev_file, batch_size, emb_type, strategy, mt_file_train_1, mt_file_dev_1,
num_epoch, eval_step, num_warmup_steps, gradient_accumulation_steps, save_dir_name, score_type):
    # initalize the process
    dist.init_process_group(backend="nccl")
    rank = int(os.environ['LOCAL_RANK'])
    # only main process initalize wandb
    if rank == 0:
        # initalize the project parameters into Wandb, store experiment specific parameters
        wandb.init(project="ContraScore", config=
        {
            "strategy": strategy,
            "epoch": num_epoch,
            "eval_step": eval_step,
            "emb_type": emb_type,
            "batch size": batch_size * gradient_accumulation_steps,
            "margin": 1 if score_type == 'L2' else 0.01,
            "score_type": score_type
        })
    exp_config.device_id = rank % torch.cuda.device_count()
    # set cuda device with rank and clear ram cache to ensure balanced ram allocations
    torch.cuda.set_device(rank)
    torch.cuda.empty_cache()
    # we will build bilingual sentence embedding models. XLM for both en-de and zh-en
    tokenizer = AutoTokenizer.from_pretrained("xlm-roberta-large-tok")
    if emb_type == 'last_layer':
        model = CL_XLM_Roberta("xlm-roberta-large-model").to(exp_config.device_id)
    elif emb_type == 'states_concat':
        model = XLM_Roberta_MLP("xlm-roberta-large-model").to(exp_config.device_id)
    else:
        print("Incorrect model type!")

    # parallelize the pipeline into multiple gpus
    optimizer = AdamW(model.parameters(), lr=exp_config.lr)
    # load in src,ref and mt data if available
    src_train_data, src_dev_data = open(src_train_file, 'r').readlines(), open(src_dev_file, 'r').readlines()
    ref_train_data, ref_dev_data = open(ref_train_file, 'r').readlines(), open(ref_dev_file, 'r').readlines()
    src_train_data, ref_train_data = [data[:-1] for data in src_train_data], [data[:-1] for data in ref_train_data]
    src_dev_data, ref_dev_data = [data[:-1] for data in src_dev_data], [data[:-1] for data in ref_dev_data]
    # if mt_file_1 is none means there is no hard negative sample
    mt1_train_data, mt1_dev_data = None, None
    if mt_file_train_1:
        mt1_train_data, mt1_dev_data = open(mt_file_train_1, 'r').readlines(), open(mt_file_dev_1, 'r').readlines()
        mt1_train_data, mt1_dev_data = [data[:-1] for data in mt1_train_data], [data[:-1] for data in mt1_dev_data]
    train_dataloader = preprocess_data(src_train_data, ref_train_data, tokenizer, exp_config.max_length, batch_size, mt1_data=mt1_train_data)
    dev_dataloader = preprocess_data(src_dev_data, ref_dev_data, tokenizer, exp_config.max_length, batch_size, mt1_data=mt1_dev_data)

    model = DDP(model, device_ids=[exp_config.device_id], find_unused_parameters=True)
    model.train()

    max_train_steps = math.ceil(len(train_dataloader) / gradient_accumulation_steps) * num_epoch
    lr_scheduler = get_linear_schedule_with_warmup(
        optimizer=optimizer,
        num_warmup_steps=num_warmup_steps,
        num_training_steps=max_train_steps,
    )

    loss_manager = loss_funct(strategy, score_type)

    with torch.autograd.set_detect_anomaly(True):
        for epoch in range(num_epoch):
            torch.cuda.empty_cache() # empty cache in gpus
            train_dataloader.sampler.set_epoch(epoch) # set the sampler at each epoch
            for step, train_batch in enumerate(train_dataloader):
                # evaluate at every eval_step and also at the end of epoch (includes the beginning loss)
                if (step % (eval_step * gradient_accumulation_steps) == 0) and rank == 0:
                    # store all the losses in wandb
                    store_cl_loss(model, dev_dataloader, loss_manager, train_batch, emb_type)

                train_loss_ls = loss_manager.loss_compute(train_batch, model, emb_type)
                train_loss = torch.stack(train_loss_ls, dim=0).sum(dim=0)

                # accumulate losses at weights, each is normalized by accumulation steps
                train_loss = train_loss / gradient_accumulation_steps
                train_loss.backward()

                if step % gradient_accumulation_steps == 0 or step == len(train_dataloader) - 1:
                    optimizer.step()
                    lr_scheduler.step()
                    optimizer.zero_grad() # clear the grads

                # save at end of epoch and at main processls
                if not os.path.isdir(f'{save_dir_name}'):
                    os.makedirs(f'{save_dir_name}')

                if step == len(train_dataloader) - 1 and rank == 0:
                    store_cl_loss(model, dev_dataloader, loss_manager, train_batch, emb_type)
                    torch.save(model.module, f'{save_dir_name}/epoch{epoch}.ckpt')
                    print(f"Saved entire model at current epoch {epoch}!")

if __name__ == "__main__":
    main()



import glob
import click

@click.command()
@click.option('-dir')
@click.option('-prefix', help="zh_CN_en_XX_")
@click.option('-save')
def main(dir, prefix, save):
    file_ls = glob.glob(f'{dir}/*')
    file_ls = sorted(file_ls, key=lambda x:int(x.split(prefix)[-1].split('_')[0]))
    new_file = open(save, 'w')
    for f_name in file_ls:
        with open(f_name, 'r') as file_ele:
            for line in file_ele:
                new_file.write(line)
    print("Files are merged!")

if __name__ == "__main__":
    main()



import click
import json
import random
from tqdm.auto import tqdm
from nltk.tokenize import word_tokenize
from nltk.tokenize.treebank import TreebankWordDetokenizer
# 1) filter out output which is identical to reference
# 2) One-path mode: For each instance, based on number of possible choices at
# each step, randomly select one path and construct data with different number
# of errors.
# 3) All-path mode: For each instance, consider all possible choices at each step
# build the entire tree path
# 4) Construct pair-wise ranking data for each instance

"""code to run: python3 syn_data/syn_data_construct.py -lang_dir zh_en -path_option one_path -data_type train"""

@click.command()
@click.option('-lang_dir')
@click.option('-path_option', type=str)
@click.option('-data_type', help='train or dev')
def main(path_option, lang_dir, data_type):
    ori_ref_ls = open(f'news_data/{lang_dir[:2]}-{lang_dir[3:]}/news_comp_{lang_dir}.{lang_dir[3:]}_{data_type}.txt', 'r').readlines()
    ori_src_ls = open(f'news_data/{lang_dir[:2]}-{lang_dir[3:]}/news_comp_{lang_dir}.{lang_dir[:2]}_{data_type}.txt', 'r').readlines()
    ref_file = open(f'news_prepare_mask_data_syn_gen/cycle1_{lang_dir}_news_train_ref_mask.txt', 'r')
    loc_dict = json.load(open(f"news_prepare_mask_data_syn_gen/cycle1_{lang_dir}_news_train_start_end_locs.json"))

    if lang_dir == 'zh_en':
        if data_type == 'train':
            out_file = open('syn_data_gen_zh_en_train_1660639001/cycle1_infill_mt_0.15_zh_en_total.txt', 'r')
        else:
            out_file = open('syn_data_gen_zh_en_dev_1660637613/cycle1_infill_mt_0.15_zh_en_total.txt', 'r')
    elif lang_dir == 'en_de':
        if data_type == 'train':
            out_file = open('syn_data_gen_en_de_train_1660639578/cycle1_infill_mt_0.15_en_de_total.txt', 'r')
        else:
            out_file = open('syn_data_gen_en_de_dev_1660638759/cycle1_infill_mt_0.15_en_de_total.txt', 'r')
    else:
        print("We only support zh-en and en-de at the moment!")

    new_loc_dict = {}
    for ref_line, out_line in zip(ref_file, out_file):
        ref_content = ref_line.split('\t')
        if ref_content[-1] != out_line:
            sen_index, err_index = int(ref_content[0]), int(ref_content[1])
            if sen_index not in new_loc_dict:
                new_loc_dict[sen_index]={}
                new_loc_dict[sen_index]['src']=ori_src_ls[sen_index][:-1]
                new_loc_dict[sen_index]['ref']=ori_ref_ls[sen_index][:-1]
                new_loc_dict[sen_index]['outputs']=[]
            new_loc_dict[sen_index]['outputs'].append([loc_dict[sen_index][err_index], out_line[:-1]])

    if path_option == 'one_path':
        one_path_dict = {}
        with tqdm(total=len(new_loc_dict)) as pbar:
            for sen_index, cur_dict in new_loc_dict.items():
                cur_ls = cur_dict['outputs']
                overall_ls = []
                while len(cur_ls) > 0:
                    # print(cur_ls)
                    overall_ls.append(cur_ls.copy())
                    rand_index = random.randrange(len(cur_ls))
                    cur_ls.pop(rand_index)

                ref_ls = word_tokenize(cur_dict['ref'])
                one_path_dict[sen_index]={}
                one_path_dict[sen_index]['src']=cur_dict['src']
                one_path_dict[sen_index]['ref']=cur_dict['ref']
                one_path_dict[sen_index]['out']=[]
                for err_sen in overall_ls:
                    last_end_ls, reconstruct_sen_ls = [], []
                    cur_sen_reconstruct = []
                    for err_index, err in enumerate(err_sen):
                        loc, err_cont = err[0], err[1]
                        err_toks = word_tokenize(err_cont)
                        if err_index == 0:
                            cur_sen_reconstruct+=ref_ls[:loc[0]]+err_toks
                        else:
                            cur_sen_reconstruct+=ref_ls[last_end_ls[-1]:loc[0]]+err_toks
                        last_end_ls.append(loc[1])
                    cur_sen_reconstruct+=ref_ls[last_end_ls[-1]:]
                    cur_sen = TreebankWordDetokenizer().detokenize(cur_sen_reconstruct)
                    one_path_dict[sen_index]['out'].append([len(err_sen), cur_sen])
                pbar.update(1)

            with open(f"instance_{lang_dir}_stratified_{data_type}_data.json", 'w') as outfile:
                json.dump(one_path_dict, outfile)

    elif path_option == 'all_path':
        pass
    else:
        print("Your provided path is incorrect!")

if __name__ == "__main__":
    random.seed(10)
    main()



from datasets import inspect_dataset, load_dataset_builder
import datasets
import click

@click.command()
@click.option('-src_lang', type=str, help="de or zh")
@click.option('-tar_lang', type=str, help="en")
def main(src_lang, tar_lang):
    # wmt19
    inspect_dataset("wmt19", "wmt19/wmt19.py")
    if src_lang == 'de':
        builder = load_dataset_builder(
            "wmt_utils.py",
            language_pair=(src_lang, tar_lang),
            subsets={
                datasets.Split.TRAIN: ["newscommentary_v14", "newsdev2014", "newsdev2015", \
                "newsdiscussdev2015", "newsdev2016", "newsdev2017", "newsdev2018", "newsdev2019", \
                "newsdiscusstest2015", "newssyscomb2009", "newstest2008", "newstest2009", \
                "newstest2010", "newstest2011", "newstest2012", "newstest2013", "newstest2014",\
                "newstest2015", "newstest2016", "newstest2017", "newstest2018"],
            },
        )
        tar = 'de'
    elif src_lang == 'zh':
        builder = load_dataset_builder(
            "wmt_utils.py",
            language_pair=(src_lang, tar_lang),
            subsets={
                datasets.Split.TRAIN: ["newscommentary_v14", "newsdev2017", "newstest2017", "newstest2018"],
            },
        )
        tar = 'en'
    else:
        print("Current language is not supported!")
    # Standard version
    builder.download_and_prepare()
    ds = builder.as_dataset()

    save_file = open(f"wmt_news/wmt_news_{src_lang}_{tar_lang}.txt", 'w')
    for line in ds['train']:
        save_file.write(line['translation'][tar]+'\n')
    print("Lines are saved!")

if __name__ == "__main__":
    main()



from nltk.tokenize import word_tokenize
from nltk.tokenize.treebank import TreebankWordDetokenizer
import random
import click
import nltk
import click
import spacy
nltk.download('punkt')
# 1) Sample five non-overlapping locations in a sentence to construct synthesized dataset
# 2) For each location, choose operations from Repeat, Delete, Swap, Insert and Replace
#    (For Replace operaiton, we use both in batch and also dictionary replacement)
# 3) Use two losses: multi-level margin losses and compactness losses

"""code to run: python3 syn_data/ssl_data_rep_insert_del_construct.py -inp_addr sep17_zh_en_ssl_data/train.en
 -rep_addr cont_total.txt -num_turns 5 -language english"""

# construct synthesized data from delete, swap, repeat, insert and replace operations
def ssl_stratified_data_schedule(words_ls, num_turns, max_span, schedule_ls, random_cont_enable):
    indices_ls = list(range(len(words_ls)))
    # dynamically store the indices that can be modified
    temp_ls = indices_ls.copy()
    # dynamically store the amount of operations that can be added on each position
    edit_queue = [len(words_ls)-ele for ele in indices_ls]

    # 0: delete, 1: insert_mt, 2: replace_mt
    operation_type = [0, 1]
    weights=[1,2]

    # randomly select operation at each turn
    ops_ls = random.choices(operation_type, weights=weights, k=num_turns)
    pre_ops_ls = []
    if sum(ops_ls) >= len(schedule_ls):
        pre_ops_ls = schedule_ls
    else:
        pre_ops_ls = random.choices(schedule_ls, k=sum(ops_ls))

    # randomly select edit position but non-overlapping to the last operation
    final_ops_ls, edit_locs, edit_conts = [], [], []
    all_cont_ls = [pre_op.split('-')[2] for pre_op in pre_ops_ls]
    if random_cont_enable:
        # randomly insert/replace content in each location
        random.shuffle(all_cont_ls)

    # format: 13-15-facilitating
    for pre_op, cur_cont in zip(pre_ops_ls, all_cont_ls):
        pre_op_cur_ls = pre_op.split('-')

        start_index, end_index = int(pre_op_cur_ls[0]), int(pre_op_cur_ls[1])
        edit_conts.append(cur_cont)
        # updates the tracking table
        for i in range(start_index-1, -1, -1):
            if edit_queue[i] == 0:
                break
            edit_queue[i] -= edit_queue[start_index]
        # for insert operation
        if start_index == end_index:
            temp_ls = list(set(temp_ls) - set([indices_ls[start_index]]))
            edit_queue[start_index] = 0
            edit_locs.append([start_index])
            final_ops_ls.append(1) # insert
        # for replace operation
        else:
            span_len = end_index-start_index
            temp_ls = list(set(temp_ls) - set(indices_ls[start_index:end_index]))
            edit_queue[start_index:end_index] = [0]*span_len
            edit_locs.append([start_index, end_index])
            final_ops_ls.append(2) # replace

    num_dels = len(ops_ls) - sum(ops_ls)
    # rest operations are for delete
    for op_index in range(num_dels):
        if len(temp_ls) > 0:
            rand_index = random.choice(temp_ls)
            for i in range(rand_index-1, -1, -1):
                if edit_queue[i] == 0:
                    break
                edit_queue[i] -= edit_queue[rand_index]

            if min(max_span, edit_queue[rand_index]) > 1:
                span_len = random.randrange(1, min(max_span, edit_queue[rand_index]))
            else:
                span_len = 1
            temp_ls = list(set(temp_ls) - set(indices_ls[rand_index:rand_index+span_len]))
            edit_queue[rand_index:rand_index+span_len] = [0]*span_len
            edit_locs.append([rand_index, rand_index+span_len])
            final_ops_ls.append(0)
        else:
            # number of turns is not supported by current sentence
            ret_ops = final_ops_ls[:sum(ops_ls)+op_index]
            return ret_ops, edit_locs, edit_conts

    return final_ops_ls, edit_locs, edit_conts

@click.command()
@click.option('-inp_addr', type=str)
@click.option('-rep_addr', type=str)
@click.option('-num_turns', type=int)
@click.option('-language', type=str, help="english or german")
@click.option('-rand_insert_replace', type=bool, help="If enabled, we randomly insert/replace span of tokens")
def main(inp_addr, rep_addr, num_turns, language, rand_insert_replace):
    max_span = 7
    lines = open(inp_addr, 'r').readlines()
    cont_lines = open(rep_addr, 'r')

    save_files_mined_ls, save_files_del_ls, save_files_mined_cont_ls = [], [], []
    num_turn_choices = list(range(1, num_turns+1))

    if language == 'japanese':
        nlp = spacy.blank('ja')
    # generate save file
    for i in range(1, num_turns+1, 1):
        save_files_mined_ls.append(open(f'mined_sen_op_loc_{i}.txt', 'w'))
    for i in range(1, num_turns+1, 1):
        save_files_del_ls.append(open(f'del_sen_op_loc_{i}.txt', 'w'))

    for i in range(1, num_turns+1, 1):
        save_files_mined_cont_ls.append(open(f'{language}_{i}_mined_cont_rand_{rand_insert_replace}.txt', 'w'))

    for cont_line_ls in cont_lines:
        temp_line_ls = cont_line_ls[:-1].split('\t')
        if len(temp_line_ls) > 1:
            cont_line_ls = temp_line_ls[1:]
            sen_index = int(temp_line_ls[0])
            line = lines[sen_index][:-1]
            cur_num_turns = random.choice(num_turn_choices)
            if language == 'japanese':
                words_ls = [word.text for word in nlp(line)]
            else:
                words_ls = word_tokenize(line)

            ops_ls, edit_locs, edit_conts = ssl_stratified_data_schedule(words_ls, cur_num_turns, max_span, \
                cont_line_ls, rand_insert_replace)
            # save as (sen_index, op, edit_locs)
            for file_index, (op, loc) in enumerate(zip(ops_ls, edit_locs)):
                if len(loc) == 1:
                    cur_loc = f'{loc[0]}'
                elif len(loc) == 2:
                    cur_loc = f'{loc[0]}_{loc[1]}'
                else:
                    print("Errors in sentence locations")
                    exit(1)

                if op == 0:
                    save_files_del_ls[file_index].write(str(sen_index)+'\t'+str(op)+'\t'+cur_loc+'\n')
                else:
                    save_files_mined_ls[file_index].write(str(sen_index)+'\t'+str(op)+'\t'+cur_loc+'\n')

            for file_index, cont in enumerate(edit_conts):
                save_files_mined_cont_ls[file_index].write(cont+'\n')

if __name__ == "__main__":
    main()



import faiss
import numpy as np
import os

lines = open('fixed_wiki_raw_2M.en', 'r').readlines()
lines = [line[:-1] for line in lines]

D = 1024
X = np.load('final.npy')
X = np.array(X, dtype='float32')
print(X.shape)

gpu_ids = "0"  # can be e.g. "3,4" for multiple GPUs
os.environ['CUDA_VISIBLE_DEVICES'] = gpu_ids

# Setup
cpu_index = faiss.IndexFlatL2(D)
gpu_index = faiss.index_cpu_to_all_gpus(cpu_index)
gpu_index.add(X)

# Search
topk = 4
dists, ids = gpu_index.search(x=X[4:5], k=topk)
print(lines[4])
print(dists)
print(ids)
for id in ids[0]:
    print(lines[id])



from nltk.tokenize import word_tokenize
from nltk.tokenize.treebank import TreebankWordDetokenizer
import random
import click
from dbdict import DbDict
import nltk
nltk.download('punkt')
# 1) Sample five non-overlapping locations in a sentence to construct synthesized dataset
# 2) For each location, choose operations from Repeat, Delete, Swap, Insert and Replace
#    (For Replace operaiton, we use both in batch and also dictionary replacement)
# 3) Use two losses: multi-level margin losses and compactness losses

"""python3 syn_data/data.py -num_gens 3 -max_span 7 -num_turns 5 -corpus_addr wiki_test_1000.en -src_lang en -trg_lang zh -enable_nsp_err True -save_name test.txt -mode single -random_num_turns False"""

# construct synthesized data from delete, swap, repeat, insert and replace operations
def ssl_stratified_data_schedule(words_ls, num_turns, max_span, enable_nsp_err=False):
    indices_ls = list(range(len(words_ls)))
    # dynamically store the indices that can be modified
    temp_ls = indices_ls.copy()
    # dynamically store the amount of operations that can be added on each position
    edit_queue = [len(words_ls)-ele for ele in indices_ls]
    # 0: delete, 1: repeat, 2: insert_nsp, 3: replace_nsp, 4: insert_dict, 5: replace_dict, 6: swap
    if enable_nsp_err:
        operation_type = [0, 1, 2, 3, 4, 5, 6]
        weights=[1,1,1,1,2,2,1]
    else:
        # delete, replace
        operation_type = [0, 5]
        weights=[1, 2]
    # randomly select operation at each turn
    ops_ls = []
    for i in range(num_turns):
        cur_op = cur_op = random.choices(operation_type, weights=weights, k=1)[0]
        # the last operation is swap
        if cur_op == 6:
            ops_ls.append([operation_type[-1], operation_type[-1]])
        else:
            ops_ls.append([cur_op])
    # randomly select edit position but non-overlapping to the last operation
    edit_locs = []
    for op_index, ops in enumerate(ops_ls):
        temp_edit_locs = []
        # this is for operation that needs two spans (like swap) in one sentence
        for op in ops:
            if len(temp_ls) > 0:
                rand_index = random.choice(temp_ls)
                for i in range(rand_index-1, -1, -1):
                    if edit_queue[i] == 0:
                        break
                    edit_queue[i] -= edit_queue[rand_index]
                if op == 2:
                    temp_edit_locs.append([rand_index])
                else:
                    if min(max_span, edit_queue[rand_index]) > 1:
                        span_len = random.randrange(1, min(max_span, edit_queue[rand_index]))
                    else:
                        span_len = 1
                    temp_ls = list(set(temp_ls) - set(indices_ls[rand_index:rand_index+span_len]))
                    edit_queue[rand_index:rand_index+span_len] = [0]*span_len
                    temp_edit_locs.append([rand_index, rand_index+span_len])
            else:
                # number of turns is not supported by current sentence
                ret_ops = ops_ls[:op_index]
                return ret_ops, edit_locs
        edit_locs.append(sorted(temp_edit_locs))
    return ops_ls, edit_locs

@click.command()
@click.option('-mode', type=str, help='single or double')
@click.option('-num_gens', default=1, type=int, help="Number of hard negative samples per instance")
@click.option('-max_span', default=7, type=int, help="Max span for each operation, less than max_span")
@click.option('-random_num_turns', type=bool, help="Whether to randomnize the number of turns")
@click.option('-num_turns', type=int, help="Number of max turns per sample")
@click.option('-corpus_addr', type=str, help="Address of pretraining corpus")
@click.option('-src_lang', type=str, default='en', help='src lang for bilingual dictionary, en')
@click.option('-trg_lang', type=str, default='zh', help='tgt lang for bilingua; dictionary, zh')
@click.option('-enable_nsp_err', type=bool, help="enable to use dict for replace and insert errors")
@click.option('-save_name', type=str, help="specify the name of the saved file")
def main(mode, corpus_addr, num_gens, max_span, random_num_turns, num_turns, src_lang, trg_lang, enable_nsp_err, save_name):
    corpus_file = open(corpus_addr, 'r')
    dict_addr = f'{src_lang}-{trg_lang}.db'
    dict_mapping = DbDict(dict_addr, True)
    save_file = open(save_name, 'w')
    save_turns_file = open('turns_'+save_name, 'w')

    fail_cases = 0
    num_turn_choices = list(range(1, num_turns+1))
    for line in corpus_file:
        line_samples, num_turns_ls = [], []
        if mode == 'single':
            words_ls = word_tokenize(line[:-1])
            enable_nsp_err = False
        elif mode == 'double':
            cur_sen, next_sen = line.split('\t')[0], line.split('\t')[1]
            words_ls, next_ls = word_tokenize(cur_sen[:-1]), word_tokenize(next_sen[:-1])
        else:
            print('Wrong specification for mode!')
            exit(1)

        # generate different spans with different roots
        for _ in range(num_gens):
            # randomly select a turn from 1, 2, 3, 4, 5
            if random_num_turns:
                num_turns = random.choice(num_turn_choices)
            # generate one root of a path
            ops_ls, edit_locs = ssl_stratified_data_schedule(words_ls, num_turns, max_span, enable_nsp_err)
            one_path_turn_per_gen_ls, one_path_sample_per_gen_ls = [], []
            # cur_count = 5
            locs_records, replaced_contents = [], []
            # 0: delete, 1: repeat, 2: insert_nsp, 3: replace_nsp, 4: insert_dict, 5: replace_dict, 6: swap
            for ops, locs in zip(ops_ls, edit_locs):
                # len(ops) == 2 implies this is a swap operation
                if len(ops) == 2:
                    # swap
                    print("no more swap operation")
                    op_span_1, op_span_2 = locs[0], locs[1]
                    locs_records.append(op_span_1)
                    locs_records.append(op_span_2)
                    replaced_contents.append(words_ls[op_span_2[0]:op_span_2[1]])
                    replaced_contents.append(words_ls[op_span_1[0]:op_span_1[1]])
                    # new_words_ls = words_ls[:op_span_1[0]] + words_ls[op_span_2[0]:op_span_2[1]] + \
                    #  words_ls[op_span_1[1]:op_span_2[0]] + words_ls[op_span_1[0]:op_span_1[1]] + words_ls[op_span_2[1]:]
                else:
                    if ops[0] == 0:
                        # delete
                        start_pos, end_pos = locs[0][0], locs[0][1]
                        locs_records.append([start_pos, end_pos])
                        replaced_contents.append([])
                        # new_words_ls = words_ls[:start_pos]+words_ls[end_pos:]
                    elif ops[0] == 1:
                        # repeat
                        start_pos, end_pos = locs[0][0], locs[0][1]
                        num_repeats = random.choice([1, 2, 3])
                        locs_records.append([start_pos, end_pos])
                        replaced_contents.append(words_ls[start_pos:end_pos] * (num_repeats+1))
                        # new_words_ls = words_ls[:start_pos] + words_ls[start_pos:end_pos] * (num_repeats+1) + words_ls[end_pos:]
                    elif ops[0] == 2 or ops[0] == 3:
                        # nsp insert or replace
                        print("I should not sample from here!")
                        # indices_ls = list(range(len(next_ls)))
                        # edit_queue = [len(words_ls)-ele for ele in indices_ls]
                        # rand_take_pos = random.choice(indices_ls)
                        # if min(max_span, edit_queue[rand_take_pos])>1:
                        #     span_len = random.randrange(1,  min(max_span, edit_queue[rand_take_pos]))
                        # else:
                        #     span_len = 1
                        # repl_cont_ls = next_ls[rand_take_pos:rand_take_pos+span_len]

                        # if ops[0] == 2:
                        #     insert_pos = locs[0][0]
                        #     new_words_ls = words_ls[:insert_pos] + repl_cont_ls + words_ls[insert_pos:]
                        # else:
                        #     start_pos, end_pos = locs[0][0], locs[0][1]
                        #     new_words_ls = words_ls[:start_pos] + repl_cont_ls + words_ls[end_pos:]
                    elif ops[0] == 4 or ops[0] == 5:
                        # dict insert or replace
                        start_pos, end_pos = locs[0][0], locs[0][1]
                        src_cont_ls, tgt_cont_ls = [], []
                        src_trg_dir, trg_src_lang = f'{src_lang}2{trg_lang}', f'{trg_lang}2{src_lang}'

                        # first translate into target language
                        for word in words_ls[start_pos:end_pos]:
                            word = word.lower()
                            word_in_trg = dict_mapping.get(f"{src_trg_dir}{word}")
                            # print(word_in_trg)
                            # literal translation to src
                            if word_in_trg:
                                probs = list(word_in_trg.values())[:-1]
                                if len(probs) != 1:
                                    probs = [1-ele for ele in probs]
                                new_sample = random.choices(list(word_in_trg.keys())[:-1], weights=probs, k=1)
                                src_cont_ls.append(new_sample[0])

                        # translate back into src language
                        for word in src_cont_ls:
                            word_in_src = dict_mapping.get(f"{trg_src_lang}{word}")
                            # literal translation to trg
                            if word_in_src:
                                probs = list(word_in_src.values())[:-1]
                                new_sample = random.choices(list(word_in_src.keys())[:-1], weights=probs, k=1)
                                tgt_cont_ls.append(new_sample[0])

                        # verity if the content is not the same
                        if len(tgt_cont_ls) > 0 and words_ls[start_pos:end_pos] != tgt_cont_ls:
                            if ops[0] == 4:
                                print("no more dict insert operation")
                                indices_ls = list(range(len(words_ls)))
                                rand_insert_pos = random.choice(indices_ls)
                                locs_records.append([rand_insert_pos, rand_insert_pos])
                                replaced_contents.append(tgt_cont_ls)
                                # new_words_ls = words_ls[:rand_insert_pos] + tgt_cont_ls + words_ls[rand_insert_pos:]
                            else:
                                locs_records.append([start_pos, end_pos])
                                replaced_contents.append(tgt_cont_ls)
                                # new_words_ls = words_ls[:start_pos] + tgt_cont_ls + words_ls[end_pos:]
                    else:
                        print("error in the operation index")
                        exit(1)

            locs_stratified_ls, conts_stratified_ls = [], []
            # generate one path out of all paths
            if len(locs_records) > 0:
                for num_errs in range(len(locs_records), 0, -1):
                    rand_inds = random.sample(range(len(locs_records)), k=num_errs)
                    locs_records = [locs_records[ind] for ind in rand_inds]
                    replaced_contents = [replaced_contents[ind] for ind in rand_inds]
                    locs_stratified_ls.append(locs_records)
                    conts_stratified_ls.append(replaced_contents)

                for locs_records, replaced_contents in zip(locs_stratified_ls, conts_stratified_ls):
                    new_words_ls = []
                    sorted_locs_cont = sorted(list(zip(locs_records, replaced_contents)))
                    if len(sorted_locs_cont) == 1:
                        new_words_ls += words_ls[:locs_records[0][0]]+replaced_contents[0]+words_ls[locs_records[0][1]:]
                    else:
                        for cur_index, locs_content in enumerate(sorted_locs_cont):
                            locs_pair, content = locs_content[0], locs_content[1]
                            if cur_index == 0:
                                new_words_ls += words_ls[:locs_pair[0]]+content
                            elif cur_index == len(sorted_locs_cont)-1:
                                new_words_ls += words_ls[sorted_locs_cont[cur_index-1][0][1]:locs_pair[0]]+content+words_ls[locs_pair[1]:]
                            else:
                                new_words_ls += words_ls[sorted_locs_cont[cur_index-1][0][1]:locs_pair[0]]+content

                    one_path_turn_per_gen_ls += [str(len(locs_records))]
                    one_path_sample_per_gen_ls += [TreebankWordDetokenizer().detokenize(new_words_ls)]
            else:
                fail_cases += 1

            # save all negative samples in one line
            line_samples.append('\t'.join(one_path_sample_per_gen_ls))
            num_turns_ls.append('\t'.join(one_path_turn_per_gen_ls))

        save_file.write(' [SEP] '.join(line_samples)+'\n')
        save_turns_file.write(' [SEP] '.join(num_turns_ls)+'\n')

    print(f"File is generated saved at {save_name}!")

if __name__ == "__main__":
    main()



import glob
import numpy as np
import click

@click.command()
@click.option('-dir_addr')
@click.option('-save_np_name')
def main(dir_addr, save_np_name):
    files_ls = glob.glob(dir_addr+'/*')
    sorted_file_ls = sorted(files_ls, key=lambda x:int(x.split('/')[1].split('_')[0]))
    total_ls =[]

    for file_name in sorted_file_ls:
        cur_ls = np.load(file_name).tolist()
        total_ls.extend(cur_ls)

    with open(save_np_name, 'wb') as f:
        np.save(f, np.array(total_ls))

if __name__ == "__main__":
    main()



import pandas as pd
import json

file_name = '/Users/challenge-2020/evaluation/human-evaluation/results/en/english_humeval_data_all_teams.json'
out_name = '/Users/challenge-2020/submissions/rdf2text/en/Amazon_AI_(Shanghai)/primary.en'
ref_name = '/Users/challenge-2020/evaluation/references/references-en.json'

out_lines = open(out_name, 'r').readlines()
out_lines = [line[:-1] for line in out_lines]
human_score_dict = json.load(open(file_name))
ref_dict = json.load(open(ref_name))

sys_score_output_dict = {}
for human_ele_dict in human_score_dict:
    if human_ele_dict['submission_id'] not in sys_score_output_dict:
        sys_score_output_dict[human_ele_dict['submission_id']] = {}
        sys_score_output_dict[human_ele_dict['submission_id']]['Correctness'] = {}
        sys_score_output_dict[human_ele_dict['submission_id']]['DataCoverage'] = {}
        sys_score_output_dict[human_ele_dict['submission_id']]['Fluency'] = {}
        sys_score_output_dict[human_ele_dict['submission_id']]['Relevance'] = {}
        sys_score_output_dict[human_ele_dict['submission_id']]['TextStructure'] = {}
        sys_score_output_dict[human_ele_dict['submission_id']]['Output'] = {}
        sys_score_output_dict[human_ele_dict['submission_id']]['Ref'] = {}
        # sys_score_output_dict[human_ele_dict['submission_id']]['Out_Refs_0'] = {}
        # sys_score_output_dict[human_ele_dict['submission_id']]['Out_Refs_1'] = {}
        # sys_score_output_dict[human_ele_dict['submission_id']]['Out_Refs_2'] = {}
        # sys_score_output_dict[human_ele_dict['submission_id']]['Out_Refs_3'] = {}

    if human_ele_dict['sample_id'] != '1124':
        sys_score_output_dict[human_ele_dict['submission_id']]['Correctness'][human_ele_dict['sample_id']] = float(human_ele_dict['Correctness'])
        sys_score_output_dict[human_ele_dict['submission_id']]['DataCoverage'][human_ele_dict['sample_id']] = float(human_ele_dict['DataCoverage'])
        sys_score_output_dict[human_ele_dict['submission_id']]['Fluency'][human_ele_dict['sample_id']] = float(human_ele_dict['Fluency'])
        sys_score_output_dict[human_ele_dict['submission_id']]['Relevance'][human_ele_dict['sample_id']] = float(human_ele_dict['Relevance'])
        sys_score_output_dict[human_ele_dict['submission_id']]['TextStructure'][human_ele_dict['sample_id']] = float(human_ele_dict['TextStructure'])
        # save all pairs of outputs and refs
        sys_score_output_dict[human_ele_dict['submission_id']]['Output'][human_ele_dict['sample_id']] = out_lines[int(human_ele_dict['sample_id'])-1]

        ref_ele_dict = ref_dict['entries'][int(human_ele_dict['sample_id'])-1][human_ele_dict['sample_id']]['lexicalisations'][0]
        sys_score_output_dict[human_ele_dict['submission_id']]['Ref'][human_ele_dict['sample_id']] = ref_ele_dict['lex']
        # sys_score_output_dict[human_ele_dict['submission_id']]['Out_Refs_0'][human_ele_dict['sample_id']] = []
        # sys_score_output_dict[human_ele_dict['submission_id']]['Out_Refs_1'][human_ele_dict['sample_id']] = []
        # sys_score_output_dict[human_ele_dict['submission_id']]['Out_Refs_2'][human_ele_dict['sample_id']] = []
        # sys_score_output_dict[human_ele_dict['submission_id']]['Out_Refs_3'][human_ele_dict['sample_id']] = []

# sanity check for the length of each system outputs
# common_segs_set = set(sys_score_output_dict['Amazon_AI_(Shanghai)']['Output'].keys())
# for sys in sys_score_output_dict:
#     common_segs_set = common_segs_set.intersection(set(sys_score_output_dict[sys]['Output'].keys()))
#     print(len(sys_score_output_dict[sys]['Output'].keys()))

with open(f'webnlg2020.json', 'w') as f:
    json.dump(sys_score_output_dict, f)

print("File is saved!")



"""This file contains all codes for inference for baseline models and ContraScore"""
"""CUDA_VISIBLE_DEVICES=0 python3 inference/inference.py -lang zh-en -wmt wmt21.news -model_addr new_margin_loss_weights/margin_src_ref_batch_last_layer_cosine_weights_1659947050 -emb_type last_layer -batch_size 250 -score L2"""

import click
from transformers import (
    AutoTokenizer,
    AutoModelForMaskedLM,
)
import torch
# from train.cl_train import preprocess_data,exp_config
from train.cl_stratified_train import preprocess_data, exp_config
from mt_metrics_eval import data
import torch.nn as nn
import json
import wandb
import torch.nn as nn
import os
import glob

human_mapping_dict = {
    "wmt21.news": {
        'en-de': ['refA', 'refD'],
        'en-ru': ['refB'],
        'zh-en': ['refA']
    },
    "wmt20": {
        'en-de': 'refb',
        'zh-en': 'refb'
    },
    "wmt21.tedtalks": {
        
    }
}

class Similarity(nn.Module):
    """
    Dot product or cosine similarity
    """

    def __init__(self, temp):
        super().__init__()
        self.temp = temp
        self.cos = nn.CosineSimilarity(dim=-1)

    def forward(self, x, y):
        return self.cos(x, y) / self.temp

class config():
    max_length = 256

def baselines_cl_eval(srcs, mt_outs_dict, refs, emb_type, model, batch_size, score, tokenizer):
    with torch.no_grad():
        # load tokenizer and models, already specified addr for tokenizer
        model.eval()
        mt_scores_dict = {'src': {}, 'ref': {}, 'src_ref': {}}
        score_funct = Similarity(temp=0.05) if score=='cosine' else nn.PairwiseDistance(p=2)
        # generate src embeddings
        for mt_name, mt_outs in mt_outs_dict.items():
            for key in mt_scores_dict:
                mt_scores_dict[key][mt_name] = []

            cur_data_dict = {'pivot': srcs, 'pos': refs, 'neg': mt_outs, 'margin': [0]*len(mt_outs)}
            cur_data_loader = preprocess_data(cur_data_dict, tokenizer, exp_config.max_length, batch_size, shuffle=False, sampler=False)
            for batch in cur_data_loader:
                # generate a batch of src, ref, mt embeddings
                pivot_pool_embeds, pos_pool_embeds, neg_pool_embeds = model(batch, emb_type)

                src_ls, ref_ls = score_funct(pivot_pool_embeds, pos_pool_embeds).tolist(), score_funct(pivot_pool_embeds, neg_pool_embeds).tolist()
                h_mean_ls = [2*src_score*ref_score/(src_score+ref_score) for src_score, ref_score in zip(src_ls, ref_ls)]
                if score == 'L2':
                    src_ls = [1/(1+score) for score in src_ls]
                    ref_ls = [1/(1+score) for score in ref_ls]
                    h_mean_ls = [1/(1+score) for score in h_mean_ls]
                mt_scores_dict['src'][mt_name].extend(src_ls)
                mt_scores_dict['ref'][mt_name].extend(ref_ls)
                mt_scores_dict['src_ref'][mt_name].extend(h_mean_ls)

        return mt_scores_dict

def corr_eval(evs, mt_scores_dict, mode, wmt, lang, outFile):
    # process the ground truth human ratings
    mqm_scores = evs.Scores(mode, 'mqm')
    qm_no_human = set(mqm_scores) - set(evs.all_refs)
    qm_human = qm_no_human.copy()
    qm_human.update(human_mapping_dict[wmt][lang])

    for eval_type, scores_dict in mt_scores_dict.items():
        print(f"Use {eval_type} for metric evaluation:", file=outFile)
        if mode == 'sys':
            # compute system-level scores (overwrite) otherwise seg scores are available already
            scores_dict = {mt_name: [sum(scores)/len(scores)] for mt_name, scores in scores_dict.items()}
        mqm_bp = evs.Correlation(mqm_scores, scores_dict, qm_human)
        mqm_bp_no = evs.Correlation(mqm_scores, scores_dict, qm_no_human)

        if mode == 'seg':
            print(f"MQM Segment-level Kendall for system + human: {mqm_bp.Kendall()[0]}", file=outFile)
            print(f"MQM Segment-level Kendall for system: {mqm_bp_no.Kendall()[0]}", file=outFile)
        elif mode == 'sys':
            print(f"MQM System-level Pearson for system + human: {mqm_bp.Pearson()[0]}", file=outFile)
            print(f"MQM System-level Pearson for system: {mqm_bp_no.Pearson()[0]}", file=outFile)
        else:
            print('Please choose between seg and sys!')
            exit(1)

@click.command()
@click.option('-lang', type=str, help="choose from zh-en, en-de and en-ru", default='zh-en')
@click.option('-wmt', type=str, help="choose from wmt21.news, wmt21.tedtalks. and wmt20", default='wmt21.news')
@click.option('-batch_size', type=int, help="batch size for each gpu")
@click.option('-score_file', type=str, help="load scores from a json file, addr of the file", default=None)
# belows are optional parameters for baseline cls
@click.option('-model_addr', default=None, help='locations of model weights')
@click.option('-emb_type', default=None, help="last_layer or states_concat")
@click.option('-score', default='L2')
def main(score_file, batch_size, lang, wmt, model_addr, emb_type, score):
    # load in data from WMT testing set
    evs = data.EvalSet(wmt, lang)
    tokenizer = AutoTokenizer.from_pretrained("xlm-roberta-large-tok")
    exp_config.device_id = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
    if score_file:
        mt_scores_dict = json.load(open(score_file))
    else:
        srcs, mt_outs_dict, refs = evs.src, evs.sys_outputs, evs.all_refs[evs.std_ref]
        if os.path.isdir(f'{model_addr}'):
            for file_addr in glob.glob(f'{model_addr}/*'):
                saveFile = open(f'{file_addr}.json', 'w')
                outFile = open(f'{file_addr}.out', 'w')
                print(f"Current evaluation is at {wmt} for {lang}: ", file=outFile)
                model = torch.load(file_addr).to(exp_config.device_id)
                mt_scores_dict = baselines_cl_eval(srcs, mt_outs_dict, refs, emb_type, model, batch_size, score)
                saveFile.write(json.dumps(mt_scores_dict))
                print(f'{file_addr}.json is saved!', file=outFile)

                # compute the segment-level correlations
                corr_eval(evs, mt_scores_dict, 'seg', wmt, lang, outFile)
                # compute the system-level correlations
                corr_eval(evs, mt_scores_dict, 'sys', wmt, lang, outFile)

if __name__ == "__main__":
    main()



import pandas as pd
import numpy as np
import json

data = pd.read_csv('zh-en_align_validation.tsv', delimiter='\t', header=None)
id_mapping_lines = open('zhen_mapping.txt', 'r').readlines()
id_mapping = {ele.split('\t')[1][:-1]: int(ele.split('\t')[0]) for ele in id_mapping_lines}
raw_id_mapping = {int(ele.split('\t')[0]): ele.split('\t')[1][:-1] for ele in id_mapping_lines}

# verify if all testing ids are valid
load_srcs_ls = open('medline_zh2en_zh.txt', 'r').readlines()[:100]
load_refs_ls = open('medline_zh2en_en.txt', 'r').readlines()
src_doc_ids = [ele.split('\t')[0]+'_'+ele.split('\t')[1] for ele in load_srcs_ls]

src_doc_ids_1000 = []
for i in range(10):
    src_doc_ids_1000 += src_doc_ids
print(len(src_doc_ids_1000))

src_id_txt_dict = {ele.split('\t')[0]+'_'+ele.split('\t')[1]: ele.split('\t')[2][:-1] for ele in load_srcs_ls}
ref_id_txt_dict = {ele.split('\t')[0]+'_'+ele.split('\t')[1]: ele.split('\t')[2][:-1] for ele in load_refs_ls}

# 1 -> raw doc id, 2 -> src seg id, 3 -> ref seg id 
src_ref_mapping_dict = {}
for check, id, src_seg_id, ref_seg_id in zip(data[0], data[1], data[2], data[3]):
    src_ref_mapping_dict[raw_id_mapping[id]+'_'+src_seg_id] = {}
    src_ref_mapping_dict[raw_id_mapping[id]+'_'+src_seg_id]['check'] = check
    src_ref_mapping_dict[raw_id_mapping[id]+'_'+src_seg_id]['ref_loc'] = raw_id_mapping[id]+'_'+ref_seg_id

# doc53->5,6; doc56->2,3,4; doc62->4,5
# print(src_ref_mapping_dict['doc53_5,6'])
# print(src_ref_mapping_dict['doc56_2,3,4'])
# print(src_ref_mapping_dict['doc62_4,5'])
# print()

data = pd.read_excel('英德_中英翻译模型评估-英语.xlsx', header=None, \
                   sheet_name=['中-英全数据-1','中-英全数据-2', '中-英全数据-3'])
ori_data = pd.read_excel('annotated_zh_en_bio_med_1000.xlsx')
out_ls = list(ori_data['translation'])

annotations = ['中-英全数据-1','中-英全数据-2', '中-英全数据-3']
scores_arr = 0
# from 1 to 4: 1->src, 2->output, 3->score, 4->annotations   
for anno in annotations:
    cur_ls = list(data[anno][3])[:-1]
    print(np.isnan(np.array(cur_ls).any()))
    scores_arr += np.array(cur_ls)
all_scores_ls = list(scores_arr/3)

src_ls, ref_ls, scores_ls, outs_ls = [], [], [], []
for ele, score, out in zip(src_doc_ids_1000, all_scores_ls, out_ls):
    if ele in src_ref_mapping_dict:
        if src_ref_mapping_dict[ele]['check'] == 'OK':
            src_ls += [src_id_txt_dict[ele]]
            doc_id = src_ref_mapping_dict[ele]['ref_loc'].split('_')[0]
            ref_seg_ids = src_ref_mapping_dict[ele]['ref_loc'].split('_')[1].split(',') 
            ele_ref_txt_ls = [ref_id_txt_dict[doc_id+'_'+id] for id in ref_seg_ids]
            ref_ls += [' '.join(ele_ref_txt_ls)]
            scores_ls += [score]
            outs_ls += [out]

bio_gt_dict = {}
bio_gt_dict['src'], bio_gt_dict['ref'], bio_gt_dict['out'], bio_gt_dict['score'] = \
src_ls, ref_ls, outs_ls, scores_ls

with open(f'gt_zh-en_biomed.json', 'w') as f:
    json.dump(bio_gt_dict, f)

print("File is saved!")



import pandas as pd

# tar -cvf - sescore_ckpt | pigz -p 8 > sescore_ckpt.gz 
# tar -xvf sescore_ckpt

labels = list(pd.read_excel('annotated_en_de_1000_second.xlsx')['score'])
labels = [str(label)+'\n' for label in labels]
# outputs = list(pd.read_excel('annotated_en_de_1000.xlsx')['translation'])
# outputs = [output+'\n' for output in outputs]
# src_lines = open('src.de', 'r').readlines()
# src_lines = [line for line in src_lines]*10
# ref_lines = open('ref.en', 'r').readlines()
# ref_lines = [line for line in ref_lines]*10

save_score = open('en_de_sec.scores', 'w')
save_score.writelines(labels)
# save_out = open('en_de.out', 'w')
# save_out.writelines(outputs)
# save_src = open('en_de.src', 'w')
# save_src.writelines(src_lines)
# save_ref = open('en_de.ref', 'w')
# save_ref.writelines(ref_lines)
# print("all the files are saved!")



from transformers import AutoTokenizer, XLMRobertaForMaskedLM
from nltk.tokenize import word_tokenize
import nltk
import torch.nn as nn
import torch
import click
import math
import time
import string
import re
import spacy
nltk.download('punkt')

"""CUDA_VISIBLE_DEVICES=0 python3 syn_data/xlm-align-measure.py -src_addr  sep17_zh_en_ssl_data/train.zh -ref_addr sep17_zh_en_ssl_data/train.en -opt_addr sep17_zh_en_ssl_data/0_5027365_news_index_table/mined_locs/mined_sen_op_loc_5.txt -gen_addr sep17_zh_en_ssl_data/0_5027365_news_index_table/cont_locs/english_5_mined_cont.txt -save_folder severity_measure_sep17_zh_en_news_0_5027365 -batch_size 75 -process_index 0 -tot_p 1 -stop_addr idf_weights/english_stopwords.txt"""

def detect_punkt(words_ls):
    for word in words_ls:
        if word not in string.punctuation:
            return False
    return True

def ret_weight_ls(sub_tgt_words_ls, stop_lines):
    if detect_punkt(sub_tgt_words_ls):
        return 'punkt'
    else:
        weight_ls=[]
        for word in sub_tgt_words_ls:
            if word.lower() in stop_lines:
                weight_ls.append(0)
            else:
                if word not in string.punctuation:
                    weight_ls.append(2)
        if max(weight_ls) < 1:
            return 'insig'
        else:
            return 'sig'

""" if it signifcant returns True """
def check_word_sig(word, stop_lines):
    if word.lower() in stop_lines:
        return False
    else:
        if word not in string.punctuation:
            return True
        else:
            return False

"""Yield batch sized list of sentences."""
def batchify(lst, batch_size):
    for i in range(0, len(lst), batch_size):
        yield lst[i:min(i+batch_size, len(lst))]

device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
model = XLMRobertaForMaskedLM.from_pretrained("xlm-align-base-model").to(device)
model.eval()
tokenizer = AutoTokenizer.from_pretrained("xlm-align-base-tok", use_fast=True, add_prefix_space=True)
max_length = 512

m = nn.Softmax(dim=-1)
mask_id = tokenizer.vocab['<mask>']
ids_to_words = {val: key for key, val in tokenizer.vocab.items()}

@click.command()
@click.option('-src_addr')
@click.option('-ref_addr')
@click.option('-opt_addr')
@click.option('-gen_addr')
@click.option('-save_folder')
@click.option('-batch_size', type=int)
@click.option("-process_index", type=int)
@click.option("-tot_p", type=int)
@click.option("-stop_addr")
@click.option("-language")
def main(src_addr, ref_addr, opt_addr, gen_addr, batch_size, save_folder, process_index, tot_p, \
        stop_addr, language): # , part_index, gpu_total
    gen_texts = open(gen_addr, 'r').readlines()
    # decide the partition of the file
    file_batch_size = math.ceil(len(gen_texts)/tot_p)
    start_end_pair = [[i, min(i+file_batch_size, len(gen_texts))] for i in range(0, len(gen_texts), file_batch_size)]
    start_index, end_index = start_end_pair[process_index][0], start_end_pair[process_index][1]
    gen_texts = gen_texts[start_index:end_index] # parition content file

    src_texts = open(src_addr, 'r').readlines()# no need to partition
    tgt_texts = open(ref_addr, 'r').readlines()# no need to partition
    stop_lines = open(stop_addr, 'r').readlines() # load all the stop words for English
    stop_lines = [line[:-1] for line in stop_lines]

    ops_loc_texts = open(opt_addr, 'r').readlines()
    ops_loc_texts = ops_loc_texts[start_index:end_index] # parition loc file

    print("file is loaded!")
    save_suffix = gen_addr.split('/')[-1]
    saveFile = open(f'{save_folder}/scores_part_{process_index}_{save_suffix}', 'w')

    new_tgt_ls, locs_ls, idf_sig_eval_ls, len_sen = [], [], [], []
    filter_line_index = []

    if language == 'japanese':
        nlp = spacy.blank('ja')

    start_time = time.time()
    for line_index, (ops_loc, gen) in enumerate(zip(ops_loc_texts, gen_texts)):
        ops_ls = ops_loc[:-1].split('\t')
        sen_index, ops_type = int(ops_ls[0]), int(ops_ls[1])
        if language == 'japanese':
            tgt_words_ls = [word.text for word in nlp(tgt_texts[sen_index][:-1])]
            gen_words_ls = [word.text for word in nlp(gen[:-1])]
        else:
            tgt_words_ls = word_tokenize(tgt_texts[sen_index][:-1])
            gen_words_ls = word_tokenize(gen[:-1])

        # insert operation
        if ops_type == 1:
            start_index = int(ops_ls[2])
            new_tgt_words_ls = tgt_words_ls[:start_index] + gen_words_ls + tgt_words_ls[start_index:]
            filter_line_index.append(line_index)
            # 0 indicates the insignicance of the original content (insert has no original content)
            idf_sig_eval_ls.append([gen_words_ls, 'insig'])
        # replace operation
        elif ops_type == 2:
            start_index, ori_end_index = int(ops_ls[2].split('_')[0]), int(ops_ls[2].split('_')[1])
            # if the new content is not the substring of the old content (remove all delete cases)
            if ' '.join(gen_words_ls) not in ' '.join(tgt_words_ls[start_index:ori_end_index]):
                new_tgt_words_ls = tgt_words_ls[:start_index] + gen_words_ls + tgt_words_ls[ori_end_index:]
                filter_line_index.append(line_index)
                # calculate the weights for the original target content
                sub_tgt_words_ls = tgt_words_ls[start_index:ori_end_index]
                idf_sig_eval_ls.append([gen_words_ls, ret_weight_ls(sub_tgt_words_ls, stop_lines), sub_tgt_words_ls])
            else:
                continue
        else:
            print("Errors in op types: ", ops_type)
            exit(1)
        end_index = start_index+len(gen_words_ls)
        new_tgt_ls.append([src_texts[sen_index][:-1]]+['</s>']+new_tgt_words_ls)
        len_sen.append(len(new_tgt_words_ls))
        locs_ls.append([start_index, end_index])
    print("Finished in: ", time.time()-start_time)

    inp_id_ls, attn_ls, emb_start_end_ls, ids_ls, final_line_index, context_parts, final_idf_sig_eval_ls = [], [], [], [], [], [], []
    count, correct = 0, 0
    # store word-wise embedding locations for all instances
    emb_locs_total_ls = []
    for sen_index, (inp_text, start_end, cur_line_index, idf_sig_eval, ele_len) in enumerate(zip(new_tgt_ls, locs_ls, filter_line_index, \
        idf_sig_eval_ls, len_sen)):
        start_index, end_index = start_end[0], start_end[1]
        ind_records = tokenizer.batch_encode_plus([inp_text], return_tensors="pt", truncation=True, \
            padding='max_length', is_split_into_words=True, max_length=max_length)

        # print(ind_records)
        # print(inp_text)
        # print(len(ind_records.tokens(0)))
        # print(ind_records.tokens(0))
        # print(ind_records.word_ids(0))
        # print()
        # exit(1)

        ind_dict, emb_locs_sublist = {}, []
        for index, ind in enumerate(ind_records.word_ids(0)):
            if ind != None:
                if ind not in ind_dict:
                    ind_dict[ind] = []
                ind_dict[ind].append(index)

        # print(inp_text)
        # print(len(ind_records.tokens(0)))
        # print(ind_records.word_ids(0))
        # print()
        # print(ind_dict)
        # print(start_index+2)
        # print(end_index+2)
        # print('----------------------')
        # end index will not be reached so only needs to +1 not plus 2!
        if max(ind_dict) >= end_index+1 and start_index != end_index and len(ind_dict) == ele_len+2:
            for cur_loc in range(start_index+2, end_index+2):
                if cur_loc in ind_dict:
                    emb_locs_sublist.append(ind_dict[cur_loc])

            emb_start, emb_end = emb_locs_sublist[0][0], emb_locs_sublist[-1][-1]+1
            emb_locs_total_ls.append(emb_locs_sublist)
            ids = [ind_records['input_ids'][:, ele_pair_index[0]:ele_pair_index[-1]+1][0].clone() \
                for ele_pair_index in emb_locs_sublist]

            ind_records['input_ids'][:, emb_start:emb_end] = mask_id
            inp_id_ls.append(ind_records['input_ids'])
            attn_ls.append(ind_records['attention_mask'])
            emb_start_end_ls.append([emb_start, emb_end])
            final_line_index.append(cur_line_index)
            correct += 1
            ids_ls.append(ids)
            context_parts.append(inp_text)
            final_idf_sig_eval_ls.append(idf_sig_eval)
        else:
            # exceed max length or not have output (equivalent to delete)
            count += 1

    print("Sentences that will be kept: ", correct)

    final_score_ls = []
    count_minor_punkt, count_minor, count_major = 0, 0, 0
    with torch.no_grad():
        for inp_id_batch, attn_batch, emb_start_end_index_batch, ids_batch, context_part_batch, idf_sig_batch, ele_emb_start_end_batch in zip( \
            batchify(inp_id_ls, batch_size), batchify(attn_ls, batch_size), batchify(emb_start_end_ls, batch_size), batchify(ids_ls, batch_size), \
            batchify(context_parts, batch_size), batchify(final_idf_sig_eval_ls, batch_size), batchify(emb_locs_total_ls, batch_size)):
            # obtain the batch of model logits
            model_dict = model(input_ids=torch.cat(inp_id_batch, dim=0).to(device), attention_mask=torch.cat(attn_batch, dim=0).to(device), return_dict=True)
            # within the batch evaluate each instance's span probabilities
            for instance_logits, emb_start_end_index, ele_ids, context_part, idf_sig, ele_emb_start_end_ls in zip(model_dict["logits"], emb_start_end_index_batch, ids_batch,\
                context_part_batch, idf_sig_batch, ele_emb_start_end_batch):
                emb_start, emb_end = emb_start_end_index[0],  emb_start_end_index[1]
                span_probs_ls = []
                # print(context_part)
                # print(idf_sig)
                # print(ele_ids)
                # for ele in ele_ids:
                #     for word in ele:
                #         print(ids_to_words[word.item()])
                for id_ls, ele_emb_start_end, word_ele in zip(ele_ids, ele_emb_start_end_ls, idf_sig[0]):
                    if check_word_sig(word_ele, stop_lines):
                        ele_start_index, ele_end_index = ele_emb_start_end[0], ele_emb_start_end[-1]+1
                        word_subtok_ls = []
                        for id, emb_probs_tensor in zip(id_ls, m(instance_logits[ele_start_index:ele_end_index, :])):
                            word_subtok_ls+=[emb_probs_tensor[id].item()]
                        #     print(emb_probs_tensor[id].item())
                        # print(sum(word_subtok_ls)/len(word_subtok_ls))
                        span_probs_ls.append(sum(word_subtok_ls)/len(word_subtok_ls))
                    # else:
                    #     print("discard one word during prob computing!")

                # if len(span_probs_ls)>0:
                #     span_prob = sum(span_probs_ls)/len(span_probs_ls)
                # else:
                #     span_prob = 1.1
                # add a corrrection step for the predefined rule
                gen_cont_result = ret_weight_ls(idf_sig[0], stop_lines)
                # For insert portion
                if len(idf_sig) == 2:
                    # if insertion is punkt, assign -0.1
                    if gen_cont_result == 'punkt':
                        final_score_ls.append(-0.1)
                        count_minor_punkt+=1
                    # if insertion is insig content, assign -1
                    elif gen_cont_result == 'insig':
                        final_score_ls.append(-1)
                    # if insertion is sig content, measure minimum word prob, if min(word span) <=0.5 -> severe
                    else:
                        if min(span_probs_ls) < 0.5:
                            final_score_ls.append(-5)
                            count_major+=1
                        else:
                            final_score_ls.append(-1)
                # For replace portion
                else:
                    # for digit difference, directly assign severe score and ignore probabilities
                    if len(idf_sig[0])==1 and len(idf_sig[2])==1 and re.sub(r'[\W\s]', ' ', idf_sig[0][0]).isdigit() \
                        and re.sub(r'[\W\s]', ' ', idf_sig[2][0]).isdigit():
                        final_score_ls.append(-5)
                        count_major+=1
                    else:
                        # if both original and replaced content are punkt, assign -0.1
                        if idf_sig[1] == 'punkt' and gen_cont_result == 'punkt':
                            final_score_ls.append(-0.1)
                            count_minor_punkt+=1
                        # if both original and replaced content are not sig, assign -1
                        elif idf_sig[1] != 'sig' and gen_cont_result != 'sig':
                            final_score_ls.append(-1)
                            count_minor+=1
                        # if replace is sig content in either original or new, measure minimum word prob, if min(word span) <=0.5 -> severe
                        else:
                            if len(span_probs_ls) > 0:
                                if min(span_probs_ls) < 0.5:
                                    final_score_ls.append(-5)
                                    count_major+=1
                                else:
                                    final_score_ls.append(-1)
                                    count_minor+=1
                            else:
                                final_score_ls.append(-5)
                                count_major+=1

                # if len(span_probs_ls)>0:
                #     print("minimum prob: ", min(span_probs_ls))
                # else:
                #     print("minimum prob: ", span_prob)
                # print("total prob: ", span_prob)
                # print("final score: ", final_score_ls[-1])
                # print('---------------------------')
                # print()

    print(count_minor_punkt)
    print(count_minor)
    print(count_major)
    # save (file line index, op type, op locs, new content, prob score)
    for cur_line_index, final_score in zip(final_line_index, final_score_ls):
        saveFile.write(ops_loc_texts[cur_line_index][:-1]+'\t'+gen_texts[cur_line_index][:-1]+'\t'+str(final_score)+'\n')

if __name__ == "__main__":
    main()



import pandas as pd
import itertools

src_ref_lines = open('ted_42604.en-ja.txt', 'r').readlines()
src_lines = [line[:-1].split('\t')[0]+'\n' for line in src_ref_lines]
ref_lines = [line[:-1].split('\t')[1]+'\n' for line in src_ref_lines]
data = pd.read_excel('和訳評価_TED_42604_eval.xlsx', index_col=0)
print(data.keys())
# remove addtional nan rows
data = data[data['原文'].notna()]
data = data[data['エラー点数'].notna()]
data = data[data['エラー点数2'].notna()]
data = data[data['エラー点数3'].notna()]
data = data[data['エラー点数4'].notna()]
fixed_src_lines, fixed_ref_lines = [], []
for index in data['原文\n番号']:
    # print(src_lines[index])
    # print(ref_lines[index])
    fixed_src_lines+=[src_lines[index]]
    fixed_ref_lines+=[ref_lines[index]]

sys_names_ls, scores_ls, final_src_ls, final_ref_ls = [], [], [], []

assert(len(list(data['System1']))==78)
sys_names_ls.extend(list(data['System1']))
assert(len(list(data['エラー点数']))==78)
scores_ls.extend(list(data['エラー点数']))
final_src_ls.extend(fixed_src_lines)
final_ref_ls.extend(fixed_ref_lines)

assert(len(list(data['System2']))==78)
sys_names_ls.extend(list(data['System2']))
assert(len(list(data['エラー点数2']))==78)
scores_ls.extend(list(data['エラー点数2']))
final_src_ls.extend(fixed_src_lines)
final_ref_ls.extend(fixed_ref_lines)

assert(len(list(data['System3']))==78)
sys_names_ls.extend(list(data['System3']))
assert(len(list(data['エラー点数3']))==78)
scores_ls.extend(list(data['エラー点数3']))
final_src_ls.extend(fixed_src_lines)
final_ref_ls.extend(fixed_ref_lines)

assert(len(list(data['System4']))==78)
sys_names_ls.extend(list(data['System4']))
assert(len(list(data['エラー点数4']))==78)
scores_ls.extend(list(data['エラー点数4']))
final_src_ls.extend(fixed_src_lines)
final_ref_ls.extend(fixed_ref_lines)

save_score = open('mqm_en_ja_scores.txt', 'w')
save_sys = open('mqm_en_ja_sys.txt', 'w')
save_src = open('mqm_en_ja_src.txt', 'w')
save_ref = open('mqm_en_ja_ref.txt', 'w')

sys_names_ls = [sys+'\n' for sys in sys_names_ls]
scores_ls = [str(score)+'\n' for score in scores_ls]

save_sys.writelines(sys_names_ls)
save_score.writelines(scores_ls)
save_src.writelines(final_src_ls)
save_ref.writelines(final_ref_ls)

print("All files are saved!")



from itertools import groupby
import click
from nltk.tokenize import word_tokenize
import spacy
from tqdm.auto import tqdm
import nltk
import math
nltk.download('punkt')

def editDistDP(str1, str2, m, n):
    # Create a table to store results of subproblems
    dp = [[{'dist': 0, 'path': [], 'mask': []} for x in range(n + 1)] for x in range(m + 1)]
    # Fill d[][] in bottom up manner
    for i in range(m + 1):
        for j in range(n + 1):
            # If first string is empty, only option is to insert all characters of second string
            if i == 0:
                dp[i][j]['dist'] = j    # Min. operations = j
                # dp[i][j]['path'] += ['insert_all']

            # If second string is empty, only option is to remove all characters of second string
            elif j == 0:
                dp[i][j]['dist'] = i    # Min. operations = i
                # dp[i][j]['path'] += ['delete_all']

            # If characters are same, ignore last char and recur for remaining string
            elif str1[i-1] == str2[j-1]:
                dp[i][j]['dist'] = dp[i-1][j-1]['dist']
                dp[i][j]['path'] = dp[i-1][j-1]['path']+['no_ops']
                dp[i][j]['mask'] += dp[i-1][j-1]['mask']+[0]

            # If last character are different, consider all possibilities and find minimum
            else:
                # insert remove replace
                if dp[i][j-1]['dist'] == min(dp[i][j-1]['dist'], dp[i-1][j]['dist'], dp[i-1][j-1]['dist']):
                    dp[i][j]['dist'] = 1 + dp[i][j-1]['dist']
                    dp[i][j]['path'] = dp[i][j-1]['path'] + [{'insert': str2[j-1]}]
                    dp[i][j]['mask'] = dp[i][j-1]['mask'] + [1]
                elif dp[i-1][j]['dist'] == min(dp[i][j-1]['dist'], dp[i-1][j]['dist'], dp[i-1][j-1]['dist']):
                    dp[i][j]['dist'] = 1 + dp[i-1][j]['dist']
                    dp[i][j]['path'] = dp[i-1][j]['path'] + [{'delete': ''}]
                    dp[i][j]['mask'] = dp[i-1][j]['mask'] + [1]
                else:
                    dp[i][j]['dist'] = 1 + dp[i-1][j-1]['dist']
                    dp[i][j]['path'] = dp[i-1][j-1]['path'] + [{'replace': str2[j-1]}]
                    dp[i][j]['mask'] = dp[i-1][j-1]['mask'] + [1]
    return dp[m][n]

@click.command()
@click.option("-pair_addr")
@click.option("-process_index", type=int)
@click.option("-tot_p", type=int)
@click.option("-language", type=str)
def main(pair_addr, process_index, tot_p, language):
    pair_lines = open(pair_addr, 'r').readlines()
    batch_size = math.ceil(len(pair_lines)/tot_p)
    start_end_pair = [[i, min(i+batch_size, len(pair_lines))] for i in range(0, len(pair_lines), batch_size)]
    start_index, end_index = start_end_pair[process_index][0], start_end_pair[process_index][1]
    pair_lines = pair_lines[start_index:end_index]

    save_file = open(f'raw_ssl_data_sep16/cont_{start_index}_{end_index}_cand_phrase.txt', 'w')

    if language == 'japanese':
        nlp = spacy.blank('ja')

    # with tqdm(total=100000) as pbar:
    for pair_line in pair_lines:
        pair_ls = pair_line[:-1].split('\t')
        sen_index, str1, str2 = pair_ls[0], pair_ls[1], pair_ls[2]
        if language == 'japanese':
            words_ls_1 = [word.text for word in nlp(str1)]
            words_ls_2 = [word.text for word in nlp(str2)]
        else:
            words_ls_1 = word_tokenize(str1)
            words_ls_2 = word_tokenize(str2)

        best_ops_path = editDistDP(words_ls_1, words_ls_2, len(words_ls_1), len(words_ls_2))
        path, mask = best_ops_path['path'], best_ops_path['mask']

        cur_start_index, real_cur_start_index, err_locs, real_err_locs = 0, 0, [], []
        for k, g in groupby(mask, lambda x:x):
            span_len = len(list(g))
            # span has to be within 6-gram
            if k == 1:
                real_span_len = 0
                check_insert = True
                for ele in path[cur_start_index:cur_start_index+span_len]:
                    if list(ele.keys())[0] != 'insert':
                        check_insert = False
                        real_span_len+=1
                if span_len <= 6 and real_cur_start_index < len(words_ls_1):
                    # when operation is insert, the whole span has to be insert
                    if check_insert:
                        err_locs.append([cur_start_index, cur_start_index+span_len, 'insert'])
                        real_err_locs.append([real_cur_start_index, real_cur_start_index+real_span_len, 'insert'])
                    else:
                        err_locs.append([cur_start_index, cur_start_index+span_len, 'others'])
                        real_err_locs.append([real_cur_start_index, real_cur_start_index+real_span_len, 'others'])
                    if real_cur_start_index >= len(words_ls_1):
                        print("Index bugs!")
                        exit(1)
                real_cur_start_index+=real_span_len
            else:
                real_cur_start_index+=span_len
            cur_start_index+=span_len


        repl_contents = []
        for err_span in err_locs:
            temp_ls = []
            for loc_index in range(err_span[0], err_span[1]):
                if list(path[loc_index].keys())[0] == 'replace':
                    temp_ls.append(path[loc_index]['replace'])
                elif list(path[loc_index].keys())[0] == 'insert':
                    temp_ls.append(path[loc_index]['insert'])
                else:
                    temp_ls.append(path[loc_index]['delete'])

            repl_contents.append(' '.join(temp_ls).strip())

        save_conts_ls=[]
        for err_loc, repl_cont in zip(real_err_locs, repl_contents):
            if repl_cont != '':
                if err_loc[2] == 'insert':
                    save_conts_ls.append('-'.join([str(err_loc[0]), str(err_loc[0])]+[repl_cont]))
                else:
                    save_conts_ls.append('-'.join([str(err_loc[0]), str(err_loc[1])]+[repl_cont]))

        if len(save_conts_ls) > 0:
            save_file.write('\t'.join([sen_index] + save_conts_ls)+'\n')
        # pbar.update(1)
    print("File is saved!")

if __name__ == "__main__":
    main()



import click
import torch
import torch.nn as nn
import wandb
from torch.utils.data import DataLoader
from datasets import Dataset
from transformers import (
    AutoTokenizer,
    AutoModel,
    DataCollatorWithPadding,
    get_linear_schedule_with_warmup,
    AdamW,
    MT5EncoderModel
)
import math
from torch.nn.parallel import DistributedDataParallel as DDP
import torch.distributed as dist
import os
import random
#from mt_metrics_eval import data
import time
from .feedforward import FeedForward
import sys

# 1) obtain the triplet pairs (ranking difference and use it to determine margins)
# 2) use margin ranking loss for all triplets, based on rankings

human_mapping_dict = {
    "wmt21.news": {
        'en-de': ['refA', 'refD'],
        'en-ru': ['refB'],
        'zh-en': ['refA']
    },
    "wmt20": {
        'en-de': ['refb'],
        'zh-en': ['refb']
    },
}

class exp_config():
    max_length = 256
    temp = 0.1
    drop_out=0.1
    activation="Tanh"
    final_activation=None

def read_ja_eval_data():
    # load in all the evaluation data
    ref_lines = open('mqm_en_ja/mqm_en_ja_ref.txt', 'r').readlines()
    ref_lines = [line[:-1] for line in ref_lines]
    out_lines = open('mqm_en_ja/mqm_en_ja_sys.txt', 'r').readlines()
    out_lines = [line[:-1] for line in out_lines]
    gt_lines = open('mqm_en_ja/mqm_en_ja_scores.txt', 'r').readlines()
    gt_lines = [float(line[:-1]) for line in gt_lines]

    gt_score_dict = {}
    gt_score_dict['sys1'] = gt_lines[:118]
    gt_score_dict['sys2'] = gt_lines[118:118*2]
    gt_score_dict['sys3'] = gt_lines[118*2:118*3]
    gt_score_dict['sys4'] = gt_lines[118*3:]
    return ref_lines, out_lines, gt_score_dict

def sent_emb(hidden_states, emb_type, attention_mask):
    if emb_type == 'last_layer':
        sen_embed = (hidden_states[-1]*attention_mask.unsqueeze(-1)).sum(1)/attention_mask.sum(-1).unsqueeze(-1)
    elif emb_type == 'avg_first_last':
        sen_embed = ((hidden_states[-1]+hidden_states[0])/2.0*attention_mask.unsqueeze(-1)).sum(1)/attention_mask.sum(-1).unsqueeze(-1)
    else:
        print(f"{emb_type} sentence emb type is not supported!")
        exit(1)
    return sen_embed

def pool(model, encoded_input, attention_mask, emb_type):
    encoded_input, attention_mask = encoded_input.to(exp_config.device_id), attention_mask.to(exp_config.device_id)
    outputs = model(input_ids=encoded_input, attention_mask=attention_mask, output_hidden_states=True)
    pool_embed = sent_emb(outputs.hidden_states, emb_type, attention_mask)
    return pool_embed

class Regression_XLM_Roberta(nn.Module):
    def __init__(self, model_addr):
        super().__init__()
        self.xlm = MT5EncoderModel.from_pretrained(model_addr)
        # initialize the feedforward to process the festures
        self.estimator = FeedForward(
            in_dim=exp_config.hidden_size * 4,
            hidden_sizes=exp_config.hidden_size_FNN,
            activations=exp_config.activation,
            dropout=exp_config.drop_out,
            final_activation=exp_config.final_activation,
        )

    def freeze_xlm(self) -> None:
        """Frezees the all layers in XLM weights."""
        for param in self.xlm.parameters():
            param.requires_grad = False

    def unfreeze_xlm(self) -> None:
        """Unfrezees the entire encoder."""
        for param in self.xlm.parameters():
            param.requires_grad = True

    def forward(self, batch, emb_type):
        pivot_pool_embed = pool(self.xlm, batch['input_ids'], batch['pivot_attn_masks'], emb_type)
        mt_pool_embed = pool(self.xlm, batch['mt_input_ids'], batch['mt_attn_masks'], emb_type)
        # compute diff between two embeds
        diff_ref = torch.abs(mt_pool_embed - pivot_pool_embed)
        prod_ref = mt_pool_embed * pivot_pool_embed
        # concatenate emebddings of mt and ref and derived features from them
        embedded_sequences = torch.cat(
            (mt_pool_embed, pivot_pool_embed, prod_ref, diff_ref), dim=1
        )
        return self.estimator(embedded_sequences)

def preprocess_data(triplets_score_dict, tokenizer, max_length, batch_size, shuffle=True, sampler=True, mode='train'):
    if mode == 'train':
        ds = Dataset.from_dict({"pivot": triplets_score_dict['pivot'], 'mt': triplets_score_dict['mt'], 'score': triplets_score_dict['score']})
    else:
        ds = Dataset.from_dict({"pivot": triplets_score_dict['pivot'], 'mt': triplets_score_dict['mt']})

    def preprocess_function(examples):
        model_inputs = {}
        # pivot examples added into dataloader, one pivot per instance
        pivot = tokenizer(examples['pivot'], max_length=max_length, padding='max_length', truncation=True)
        model_inputs['input_ids'], model_inputs['pivot_attn_masks'] = pivot['input_ids'], pivot['attention_mask']
        # mt examples added into dataloader, one mt per instance
        mt = tokenizer(examples['mt'], max_length=max_length, padding='max_length', truncation=True)
        model_inputs['mt_input_ids'], model_inputs['mt_attn_masks'] = mt["input_ids"], mt['attention_mask']
        # store the labels in model inputs
        if mode == 'train':
            model_inputs['score'] = examples['score']
        return model_inputs

    processed_datasets = ds.map(
        preprocess_function,
        batched=True,
        num_proc=None,
        remove_columns=ds.column_names,
        desc="Running tokenizer on dataset",
    )

    data_collator = DataCollatorWithPadding(
        tokenizer=tokenizer,
        padding='max_length',
        max_length=max_length,
        pad_to_multiple_of=None,
        return_tensors = 'pt'
    )

    if sampler:
        data_sampler = torch.utils.data.distributed.DistributedSampler(processed_datasets, shuffle=shuffle)
        dataloader = DataLoader(processed_datasets, batch_size=batch_size, collate_fn=data_collator, sampler=data_sampler)
    else:
        dataloader = DataLoader(processed_datasets, batch_size=batch_size, collate_fn=data_collator, shuffle=shuffle)
    return dataloader

class loss_funct():
    def __init__(self):
        # be cautiously when applying cosine because order of pos neg will be opposite (targets=1)
        self.loss_fct = nn.MSELoss()

    # If src_based, ref will be used as pos samples (pos:0, batchNeg: 6)->max_diff=6. If ref_based, max_diff=5
    def loss_compute(self, batch, model, emb_type):
        score = model(batch, emb_type)
        return self.loss_fct(score, batch['score'].unsqueeze(1).to(exp_config.device_id))

def baselines_cl_eval(mt_outs_dict, refs, emb_type, model, batch_size, tokenizer):
    with torch.no_grad():
        # load tokenizer and models, already specified addr for tokenizer
        mt_scores_dict = {}
        # generate src embeddings
        for mt_name, mt_outs in mt_outs_dict.items():
            mt_scores_dict[mt_name] = []
            cur_data_dict = {'pivot': refs, 'mt': mt_outs}
            cur_data_loader = preprocess_data(cur_data_dict, tokenizer, exp_config.max_length, batch_size, shuffle=False, sampler=False, mode='test')
            for batch in cur_data_loader:
                # generate a batch of ref, mt embeddings
                score = model(batch, emb_type).squeeze(1).tolist()
                mt_scores_dict[mt_name].extend(score)
        return mt_scores_dict

def baselines_cl_eval_en_ja(mt_outs, refs, emb_type, model, batch_size, tokenizer):
    with torch.no_grad():
        mt_score_dict = {}
        seg_scores = []
        cur_data_dict = {'pivot': refs, 'mt': mt_outs}
        cur_data_loader = preprocess_data(cur_data_dict, tokenizer, exp_config.max_length, batch_size, shuffle=False, sampler=False, mode='test')
        for batch in cur_data_loader:
            # generate a batch of ref, mt embeddings
            score = model(batch, emb_type).squeeze(1).tolist()
            seg_scores.extend(score)

        mt_score_dict['sys1'] = seg_scores[:118]
        mt_score_dict['sys2'] = seg_scores[118:118*2]
        mt_score_dict['sys3'] = seg_scores[118*2:118*3]
        mt_score_dict['sys4'] = seg_scores[118*3:]
        return mt_score_dict

def store_regression_loss(model, loss_manager, train_batch, emb_type):
    with torch.no_grad():
        save_dict = {}
        train_loss = loss_manager.loss_compute(train_batch, model, emb_type)
        save_dict['total_train_loss'] = train_loss.item()
    return save_dict

def store_corr_eval_en_ja(evs, mt_scores_dict, gt_scores_dict):
    save_dict = {}
    mqm_bp = evs.Correlation(gt_scores_dict, mt_scores_dict, gt_scores_dict.keys())
    save_dict['seg']=mqm_bp.Kendall()[0]
    return mqm_bp.Kendall()[0], save_dict

def store_corr_eval_ted(evs, mt_scores_dict, mode, lang):
    mqm_scores = evs.Scores(mode, 'mqm')
    qm_no_human = set(mqm_scores) - set(evs.all_refs)

    save_dict = {}
    mqm_bp_no = evs.Correlation(mqm_scores, mt_scores_dict, qm_no_human)
    save_dict['seg_ted']=mqm_bp_no.Kendall()[0]
    return mqm_bp_no.Kendall()[0], save_dict

def store_corr_eval(evs, mt_scores_dict, mode, wmt, lang):
    # process the ground truth human ratings
    mqm_scores = evs.Scores(mode, 'mqm')
    qm_no_human = set(mqm_scores) - set(evs.all_refs)
    qm_human = qm_no_human.copy()
    qm_human.update(human_mapping_dict[wmt][lang])
    save_dict, temp_ls = {}, []

    if mode == 'sys':
        # compute system-level scores (overwrite) otherwise seg scores are available already
        mt_scores_dict = {mt_name: [sum(scores)/len(scores)] for mt_name, scores in mt_scores_dict.items()}
    mqm_bp = evs.Correlation(mqm_scores, mt_scores_dict, qm_human)
    mqm_bp_no = evs.Correlation(mqm_scores, mt_scores_dict, qm_no_human)

    if mode == 'seg':
        save_dict['seg_system_human']=mqm_bp.Kendall()[0]
        save_dict['seg_system']=mqm_bp_no.Kendall()[0]
        temp_ls.append(mqm_bp.Kendall()[0])
    elif mode == 'sys':
        save_dict['sys_system_human']=mqm_bp.Pearson()[0]
        save_dict['sys_system']=mqm_bp_no.Pearson()[0]
        temp_ls.append(mqm_bp.Pearson()[0])
    else:
        print('Please choose between seg and sys!')
        exit(1)
    return max(temp_ls), save_dict


@click.command()
@click.option('-lr', type=float, help="learning rate", default=1e-5)
@click.option('-lang_dir', type=str, help="en_de or zh_en", default="zh_en")
@click.option('-gradient_accumulation_steps', default=1, type=int)
@click.option('-batch_size', type=int, help="train and eval batch size for contrastive learning", default=40)
@click.option('-emb_type', type=str, help="choose from last_layer, avg_first_last and states_concat", default="last_layer")
@click.option('-num_epoch', type=int, help="Number of epoches to train", default=1)
@click.option('-eval_step', type=int, help="Number of steps to evaluate", default=400)
@click.option('-num_warmup_steps', type=int, help="Number of steps to warm up", default=0)
@click.option('-save_dir_name', type=str, help="the dir name of weights being saved", default=None)
@click.option('-enable_loss_eval', type=bool, help="If given true, we will evaluate loss per eval step", default=False)
@click.option('-data_file', type=str, help="fixed_wmt_news_cl_10_en_zh_rand_True_5.txt", default="fixed_wmt_news_cl_10_en_zh_rand_True_5.txt")
@click.option('-gradual_unfrozen', type=bool, help="Specify whether xlm needs to be frozen at beginning")
@click.option('-model_base', type=str, help="Specify which model base we should use: rembert or xlm-roberta-large")
@click.option('-hidden_size', type=int, help="model base's hidden size dim: 1024 or 1152")
@click.option('-model_addr', type=str, help="The addr of the model weight")
@click.option('-load_file_enable', type=bool)
@click.option('-test_ted_enable', type=bool)
@click.option('-alpha', type=float)
def main(lang_dir, gradient_accumulation_steps, batch_size, emb_type, num_epoch, eval_step, \
            num_warmup_steps, save_dir_name, enable_loss_eval, data_file, gradual_unfrozen, lr, model_base, hidden_size, \
            model_addr, load_file_enable, test_ted_enable, alpha):
    # load in eval data
    exp_config.lr = lr
    exp_config.hidden_size = hidden_size
    exp_config.hidden_size_FNN=[hidden_size*2, hidden_size]
    wmt, lang = 'wmt21.news', lang_dir.replace('_', '-')
    evs = data.EvalSet(wmt, lang)
    if lang == 'en-ja':
        ref_lines, out_lines, gt_score_dict = read_ja_eval_data()
    else:
        mt_outs_dict, refs = evs.sys_outputs, evs.all_refs[evs.std_ref]
        evs_ted = data.EvalSet('wmt21.tedtalks', lang)
        mt_outs_dict_ted, refs_ted = evs_ted.sys_outputs, evs_ted.all_refs[evs_ted.std_ref]
    # initalize the process
    dist.init_process_group(backend="nccl")
    rank = int(os.environ['LOCAL_RANK'])
    # only main process initalize wandb
    if rank == 0:
        # initalize the project parameters into Wandb, store experiment specific parameters
        wandb.init(project="ContraScore", config=
        {
            "strategy": "Regression",
            "epoch": num_epoch,
            "eval_step": eval_step,
            "emb_type": emb_type,
            "train batch size": batch_size * gradient_accumulation_steps * 8,
            "lr": exp_config.lr,
        })

    exp_config.device_id = rank % torch.cuda.device_count()
    # set cuda device with rank and clear ram cache to ensure balanced ram allocations
    torch.cuda.set_device(rank)
    torch.cuda.empty_cache()
    # we will build bilingual sentence embedding models. XLM for both en-de and zh-en
    tokenizer = AutoTokenizer.from_pretrained(f"{model_base}-tok")

    if load_file_enable:
        model = torch.load(model_addr, map_location="cpu").to(exp_config.device_id)
    else:
        if emb_type == 'last_layer':
            model = Regression_XLM_Roberta(f"{model_base}-model").to(exp_config.device_id)
        else:
            print("Incorrect model type!")
            exit(1)

    # parallelize the pipeline into multiple gpus
    optimizer = AdamW(model.parameters(), lr=exp_config.lr)

    data_lines = open(data_file, 'r').readlines()
    ref_ls, mt_ls, score_ls = [], [], []
    data_lines = [ele[:-1] for ele in data_lines]
    for line in data_lines:
        line_ls = line.split('\t')
        ref, mt, score = line_ls[0], line_ls[1], line_ls[2]
        ref_ls += [ref]
        mt_ls += [mt]
        score_ls += [float(score)]

    triplets_score_train_dict = {'pivot': ref_ls, 'mt': mt_ls, 'score': score_ls}
    train_dataloader = preprocess_data(triplets_score_train_dict, tokenizer, exp_config.max_length, batch_size, shuffle=True, sampler=True, mode='train')

    model = DDP(model, device_ids=[exp_config.device_id], find_unused_parameters=True)
    model.train()

    max_train_steps = math.ceil(len(train_dataloader) / gradient_accumulation_steps) * num_epoch
    lr_scheduler = get_linear_schedule_with_warmup(
        optimizer=optimizer,
        num_warmup_steps=num_warmup_steps,
        num_training_steps=max_train_steps,
    )

    loss_manager = loss_funct()

    # save at end of epoch and at main processls
    if not os.path.isdir(f'{save_dir_name}') and rank == 0:
        os.makedirs(f'{save_dir_name}')

    global_step = 0 # monitor the overall steps to decide to unfreeze xlm layers
    if gradual_unfrozen:
        model.module.freeze_xlm() # freeze the xlm parameters

    with torch.autograd.set_detect_anomaly(True):
        for epoch in range(num_epoch):
            # reset the best correlation each time at start of epoch
            cur_best_cor = float('-inf')
            torch.cuda.empty_cache() # empty cache in gpus
            train_dataloader.sampler.set_epoch(epoch) # set the sampler at each epoch
            for step, train_batch in enumerate(train_dataloader):
                # reset the best correlation at 25 * eval steps
                if (step % (eval_step * gradient_accumulation_steps * 5) == 0):
                    cur_best_cor = float('-inf')
                # evaluate at every eval_step and also at the end of epoch (includes the beginning loss)
                if ((step % (eval_step * gradient_accumulation_steps) == 0) or (step == len(train_dataloader) - 1)) and rank == 0:
                    # store all the losses in wandb
                    print("start to evaluate!")
                    model.eval()
                    wandb_temp_dict = {}
                    # evaluate on the seg and sys correlations
                    start_eval_time = time.time()
                    if lang == 'en-ja': # evs, mt_scores_dict, gt_scores_dict
                        mt_scores_dict = baselines_cl_eval_en_ja(out_lines, ref_lines, emb_type, model, 200, tokenizer)
                        step_seg_cor, save_seg_dict = store_corr_eval_en_ja(evs, mt_scores_dict, gt_score_dict)
                        wandb_temp_dict.update(save_seg_dict)
                    else:
                        mt_scores_dict = baselines_cl_eval(mt_outs_dict, refs, emb_type, model, 200, tokenizer)
                        step_seg_cor, save_seg_dict = store_corr_eval(evs, mt_scores_dict, 'seg', wmt, lang)
                        _, save_sys_dict = store_corr_eval(evs, mt_scores_dict, 'sys', wmt, lang)
                        wandb_temp_dict.update(save_seg_dict)
                        wandb_temp_dict.update(save_sys_dict)
                        if test_ted_enable:
                            mt_scores_dict_ted = baselines_cl_eval(mt_outs_dict_ted, refs_ted, emb_type, model, 200, tokenizer)
                            step_seg_cor_ted, save_seg_dict_ted = store_corr_eval_ted(evs_ted, mt_scores_dict_ted, 'seg', lang)
                            wandb_temp_dict.update(save_seg_dict_ted)
                    # monitor the training loss
                    if enable_loss_eval:
                        save_loss_dict = store_regression_loss(model, loss_manager, train_batch, emb_type)
                        wandb_temp_dict.update(save_loss_dict)
                    wandb.log(wandb_temp_dict)
                    print("Testing Duration: ", time.time()-start_eval_time)
                    # save at the best epoch step
                    if step_seg_cor > cur_best_cor:
                        cur_best_cor=step_seg_cor
                        torch.save(model.module, f'{save_dir_name}/epoch{epoch}_best_{step}.ckpt')
                        print(f"Saved best model at current epoch {epoch}!")
                    model.train()
                    torch.cuda.empty_cache()

                if gradual_unfrozen and (global_step == int(len(train_dataloader)*alpha)):
                    # unfreeze the model paramters after the first evaluation step
                    model.module.unfreeze_xlm()

                train_loss = loss_manager.loss_compute(train_batch, model, emb_type)

                # accumulate losses at weights, each is normalized by accumulation steps
                train_loss = train_loss / gradient_accumulation_steps
                train_loss.backward()

                if step % gradient_accumulation_steps == 0 or step == len(train_dataloader) - 1:
                    optimizer.step()
                    lr_scheduler.step()
                    optimizer.zero_grad() # clear the grads

                global_step += 1

if __name__ == "__main__":
    random.seed(10)
    main()
